/*= R&S SGU Signal Generator ================================================
* LabWindows/CVI 9.1.0 VXIpnp Instrument Driver
* Original Release: September 2010
* By: Petr Kominek, Jiri Kominek
*
* Should you have any technical questions please contact the hotline of
* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
*
* e-mail: CustomerSupport@rohde-schwarz.com
*
*   see ChangeLog
*
*===========================================================================*/
#ifdef _WIN32
#include <windows.h>
#endif
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "rssgu.h"
#include "rssgu_utility.h"
#include <limits.h>
#include <locale.h>  
#include "visaabstraction.h"
#include "SgmaCommandIds.h"

/*= Macros ==================================================================*/

#define CHECKERR_CU(fCal) if (rssgu_status = (fCal), rssgu_status < VI_SUCCESS) \
return rssgu_initCleanUp (rmSession, instrSession, rssgu_status); else

/*= DEFINES =================================================================*/
#define RSSGU_REVISION          "Rev 1.5.0, 09/2015, CVI 2015" /* Instrument driver revision */
#define BUFFER_SIZE             2048L    /* File I/O buffer size */

#define RSSGU_DEFAULT_TIMEOUT   10000   /* Default Timeout value in ms (10 sec) */

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString instrVersionList[] = {"SGU", VI_NULL};
static ViString LevelModeArr[] = {"NORM", "LOWN", "LOWD", VI_NULL};
static ViString LevelSettingCharacteristicArr[] = {"AUTO", "UNIN", "MON", "CVSW", "USER", VI_NULL};
static ViString LevelPowerOnStateArr[] = {"OFF", "UNCH", VI_NULL};
static ViString LevelALCStateArr[] = {"ONT", "1", "OFFT", VI_NULL};
static ViString RemoteOperationModeArr[] = {"EXT", "STD", VI_NULL};
static ViString AngleUnitsArr[] = {"DEG", "RAD", VI_NULL};
static ViString PowerUnitsArr[] = {"V", "DBM", VI_NULL};
//static ViString VelocityUnitsArr[] = {"MPS", "KMH","MPH", VI_NULL};
static ViString ConfigureNetworkIPArr[] = {"AUTO", "STAT", VI_NULL};
static ViString attenuatorAfterArr[] = {"FIX","MAX", VI_NULL};
static ViString attenuatorModeArr[] = {"AUTO","FIX", VI_NULL};
static ViString terminatorModeArr[] = {"EOI","STAN", VI_NULL};
static ViString formatDataArr[] = {"ASC","PACK", VI_NULL};
static ViString regGroupArr[] = {"OPER","QUES", VI_NULL};
static ViString regTypeArr[] = {"COND","EVEN","ENAB","NTR","PTR", VI_NULL};
static ViString testResult[] = {"0", "1","RUN","STOP", VI_NULL};
static ViString trigConOpModeArr[] = {"IN", "LOW", "MLAT", "MARR", "HIGH", "FBF", "SVAL", "EO1", "EO2", "BLAN", "SNV", "PEMS", VI_NULL};
static ViString mmacBarArr[] = {"SIGMACOM", "IQUPCON20", VI_NULL};
static ViString ecoModeArr[] = {"OFF", "EM1", VI_NULL};
static ViString pulmPolArr[] = {"NORM", "INV", VI_NULL};
static ViString SerialBaudRateArr[] = {"2400", "4800", "9600", "19200", "38400", "57600", "115200", VI_NULL};
static ViString SerialParityArr[] = {"NONE", "ODD", "EVEN", VI_NULL};
static ViString SerialStopBitsArr[] = {"1", "2", VI_NULL};

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViStatus rssgu_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
                            ViStatus currentStatus);
ViStatus rssgu_defaultInstrSetup (ViSession openInstrSession);





/*****************************************************************************
* Function: Initialize
* Purpose:  This function performs the following initialization actions:
*           
*           - Opens a session to the Default Resource Manager resource and a 
*           session to the specified device using the interface and address 
*           specified in the Resource_Name control.
*           
*           - Performs an identification query on the Instrument.
*           
*           - Resets the instrument to a known state.
*           
*           - Sends initialization commands to the instrument that set any 
*           necessary programmatic variables such as Headers Off, Short 
*           Command form, and Data Transfer Binary to the state necessary for 
*           the operation of the instrument driver.
*           
*           - Returns an Instrument Handle which is used to differentiate 
*           between different sessions of this instrument driver.
*           
*           - Each time this function is invoked a Unique Session is opened.  
*           It is possible to have more than one session open for the same 
*           resource.
*           
*****************************************************************************/

ViStatus _VI_FUNC rssgu_init(
                             ViRsrc  resourceName,
                             ViBoolean   idQuery,
                             ViBoolean   resetDevice,
                             ViSession*  instrSession
                             )
{
  ViStatus            rssgu_status = VI_SUCCESS;
  ViSession           rmSession = 0;
  ViChar              buffer[BUFFER_SIZE] = "";
  rssgu_instrRange    instrPtr = NULL;
  unsigned int        ptrHandle;

  /* --- Set standard "C" locale --- */
  {
    char *locale;

    locale = setlocale(LC_ALL, "C");
  }

  /*- Open instrument session ---------------------------------------------*/
  CHECKERR( rsOpenDefaultRM (&rmSession));

  if ((rssgu_status = rsOpen (rmSession, resourceName, VI_LOAD_CONFIG,
    VI_NULL, instrSession)) < 0)
  {
    (void)rsClose (rmSession);
    return rssgu_status;
  }

  /* Determine if the structure has been initialized for the current VISA  */
  /* Session and malloc if it has not.                                     */

  instrPtr = (rssgu_instrRange)calloc (1, sizeof (struct rssgu_statusDataRanges));
  ptrHandle = rssgu_AddPointer(instrPtr);       


  CHECKERR( rsSetAttribute (*instrSession, VI_ATTR_USER_DATA, ptrHandle));

  //With remote control via Ethernet, remote control is not automatically
  //set by means of a command. The instrument must be explicitly set to
  //the REMOTE state, e.g. by sending the intf command &GTR
  CHECKERR (rsPrintf (*instrSession, "&GTR\n"));

  /*- Configure VISA Formatted I/O ----------------------------------------*/
  CHECKERR( rsSetAttribute (*instrSession, VI_ATTR_TMO_VALUE,
    RSSGU_DEFAULT_TIMEOUT));
  CHECKERR( rsSetBuf (*instrSession, (ViUInt16) (VI_READ_BUF|VI_WRITE_BUF), 4000));
  CHECKERR( rsSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
    VI_FLUSH_ON_ACCESS));
  CHECKERR( rsSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
    VI_FLUSH_ON_ACCESS));

  /* -------------------------------------------------------------------- */
  /* Drawback of Agilent VISA is patched by adding extra termination      */
  /* character "\n" to each I/O operation                                 */
  /*                                                                      */
  /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE     */
  /* -------------------------------------------------------------------- */

  CHECKERR( rsSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));

  // read/reset ESR which may still indicate device dependent error from 
  // previous handling!
  (void) rssgu_checkStatus(*instrSession);

  /*- Identification Query ------------------------------------------------*/
  if (idQuery == VI_TRUE)
  {
    int i = 0;
    CHECKERR( rsQueryf (*instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]",
      buffer));

    /* Check if instrument is supported or not */
    while (instrVersionList[i] != NULL)
    {
      if (strncmp (buffer, instrVersionList[i], strlen (instrVersionList[i])) == 0)
        break;
      else i++;
    }
    if (instrVersionList[i] == NULL)
      return rssgu_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    else
      instrPtr->instrVer = i;
  }
  /* Check for instruments options */
  if ((rssgu_status = rssgu_OptionIdentificationQuery (*instrSession, buffer)) < VI_SUCCESS)
    return rssgu_status;
  else
    rssgu_status = VI_SUCCESS;

  instrPtr->instrOptions.B26 =    (strstr (buffer, "SGU-B26") != VI_NULL) ? 1 : 0;
  instrPtr->instrOptions.B120 =   (strstr (buffer, "SGU-B120") != VI_NULL) ? 1 : 0;
  instrPtr->instrOptions.B120V =  (strstr (buffer, "SGU-B120V") != VI_NULL) ? 1 : 0;
  instrPtr->instrOptions.B140 =   (strstr (buffer, "SGU-B140") != VI_NULL) ? 1 : 0;
  instrPtr->instrOptions.B140V =  (strstr (buffer, "SGU-B140V") != VI_NULL) ? 1 : 0;

  /*- Reset instrument ----------------------------------------------------*/
  if (resetDevice == VI_TRUE)
  {
    CHECKERR( rssgu_reset (*instrSession));
  }
  else  /*- Send Default Instrument Setup ---------------------------------*/
  {
    CHECKERR( rssgu_defaultInstrSetup (*instrSession));
  }
  return rssgu_status;
}

/// HIFN This function enables and disables special PCI express features.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR enableFast/Activates/deactivates the fast mode.
/// HIPAR enableAsynchronous/Activates/deactivates the asynchronous mode.
ViStatus _VI_FUNC rssgu_UseFastSettings (ViSession instrSession,ViBoolean enableFast, ViBoolean enableAsynchronous)
{
  ViStatus rssgu_status = rsSetAttribute  (instrSession, SGS_ATTR_FAST, enableFast);
  if (rssgu_status == VI_SUCCESS && enableFast)
    rssgu_status = rsSetAttribute  (instrSession, SGS_ATTR_FASTASYNCHRONOUS, enableAsynchronous);

  if (rssgu_status == VI_SUCCESS)
    CHECKERR( rssgu_checkStatus (instrSession));

  return rssgu_status;
}

/// HIFN This function returns the states of parameters PCIe enable and PCIe asynchronous mode enable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR enableFast/Returns the state of the fast mode.
/// HIPAR enableAsynchronous/Returns the state of the asynchronous mode.
ViStatus _VI_FUNC rssgu_GetFastSettings (ViSession instrSession,ViPBoolean enableFast, ViPBoolean enableAsynchronous)
{
  ViStatus rssgs_status = rsGetAttribute  (instrSession, SGS_ATTR_FAST, enableFast);

  if ((rssgs_status == VI_SUCCESS) && (enableFast != VI_FALSE))
  {
    //ViBoolean fastEnabled  = VI_FALSE;
    rssgs_status = rsGetAttribute  (instrSession, SGS_ATTR_FASTASYNCHRONOUS, enableAsynchronous);
  }
  return rssgs_status;
}


/// HIFN In fast mode this function sends an empty command to instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_FastSync (ViSession instrSession)
{
  ViStatus rssgu_status = rsFastSync (instrSession);
  if (rssgu_status == VI_ERROR_NSUP_OPER)
  {
    ViInt32 OPCState = 0;
    rssgu_status = rssgu_OperationCompleteQuery (instrSession, &OPCState);
  }
  return rssgu_status;
}


//=====================================================================================
/// HIFN This function sets the state of the signal output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/This control sets the state of the signal output.
ViStatus _VI_FUNC rssgu_SetRFState (ViSession instrSession, ViBoolean state)
{
  ViUInt32 supported = 0;
  viFastSettingSupported (instrSession, IdSguRfState, &supported);
  if (supported)
    return viFastSetInt32(instrSession, IdSguRfState, state);

  return rssgu_setViBoolean ( instrSession,
    ":OUTP",
    state,
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the state of the signal output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/Displays the state of the signal output.
ViStatus _VI_FUNC rssgu_GetRFState (ViSession instrSession, ViBoolean *state)
{
  ViUInt32 supported = 0;
  viFastQuerySupported(instrSession, IdSguRfState, &supported);
  if (supported)
  {
    ViInt32 sel = 0;
    int status = 0;
    status = viFastQueryInt32(instrSession, IdSguRfState, &sel);
    *state = (sel != 0);
    return status;
  }

  return rssgu_getViBoolean ( instrSession,
    ":OUTP?",
    state,
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the number of RF paths.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR paths/Displays the number of RF paths.
ViStatus _VI_FUNC rssgu_GetRFNumberofPaths (ViSession instrSession,
                                            ViInt32 *paths)
{
  return rssgu_getViInt32 (instrSession,
    ":SOUR:PATH:COUN?",
    paths,
    VI_ERROR_PARAMETER2);
}

/// HIFN This function sets the mode of the attenuator at the RF output (Attenuator MODe).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR attenuatorMode/This control sets the mode of the attenuator at the RF output (Attenuator MODe).
ViStatus _VI_FUNC rssgu_SetRFAttenuatorMode (ViSession instrSession,
                                             ViInt32 attenuatorMode)
{
  return rssgu_setEnum (instrSession,
    "OUTP:AMOD",
    attenuatorMode,
    attenuatorModeArr,
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the mode of the attenuator at the RF output (Attenuator MODe).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR attenuatorMode/Displays the mode of the attenuator at the RF output (Attenuator MODe).
ViStatus _VI_FUNC rssgu_GetRFAttenuatorMode (ViSession instrSession,
                                             ViInt32 *attenuatorMode)
{
  return rssgu_getEnum (instrSession,
    "OUTP:AMOD?",
    attenuatorMode,
    attenuatorModeArr,
    VI_ERROR_PARAMETER2);
}



//=====================================================================================
/// HIFN This function sets the RF frequency of the RF output connector
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR frequency/Sets the RF frequency of the RF output connector.
ViStatus _VI_FUNC rssgu_SetFrequency (ViSession instrSession,
                                      ViReal64 frequency)
{

  ViUInt32 supported = 0;
  viFastSettingSupported (instrSession, IdSguFreq, &supported);
  if (supported)
    return viFastSetReal64(instrSession, IdSguFreq, frequency);

  return rssgu_setViReal64 (instrSession,
    ":FREQ",
    frequency,
    10.0e+6, 40e+9,
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the RF frequency of the RF output connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR frequency/Displays the RF frequency of the RF output connector.
ViStatus _VI_FUNC rssgu_GetFrequency (ViSession instrSession,
                                      ViReal64 *frequency)
{
  ViUInt32 supported = 0;
  viFastQuerySupported (instrSession, IdSguFreq, &supported);
  if (supported)
    return viFastQueryReal64(instrSession, IdSguFreq, frequency);

  return rssgu_getViReal64 (instrSession,
    ":FREQ?",
    frequency,
    VI_ERROR_PARAMETER2);
}


//=====================================================================================
/// HIFN This function queries the frequency of the local oscillator input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR frequency/Displays the frequency of the local oscillator input signal.
ViStatus _VI_FUNC rssgu_GetLoscFrequency (ViSession instrSession,
                                          ViReal64 *frequency)
{
  ViUInt32 supported = 0;
  viFastQuerySupported(instrSession, IdSguHostFreq, &supported);
  if (supported)
    return viFastQueryReal64(instrSession, IdSguHostFreq, frequency);

  return rssgu_getViReal64 (instrSession,
    ":SOUR:LOSC:FREQ?",
    frequency,
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN his function queries the level of the local oscillator input signal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR level/Displays the level of the local oscillator input signal
ViStatus _VI_FUNC rssgu_GetLoscLevel (ViSession instrSession, ViReal64 *level)
{
  ViUInt32 supported = 0;
  viFastQuerySupported(instrSession, IdSguHostPow, &supported);
  if (supported)
    return viFastQueryReal64(instrSession, IdSguHostPow, level);

  return rssgu_getViReal64 (instrSession, 
    ":SOUR:LOSC:POW?", 
    level, 
    VI_ERROR_PARAMETER2);
}  

//=====================================================================================
/// HIFN This function sets the RF signal's power level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR level/This control sets the RF signal's power level.
ViStatus _VI_FUNC rssgu_SetLevel (ViSession instrSession, ViReal64 level)
{

  ViUInt32 supported = 0;
  viFastSettingSupported (instrSession, IdSguPow, &supported);
  if (supported)
    return viFastSetReal64(instrSession, IdSguPow, level);

  return rssgu_setViReal64 (instrSession, 
    ":POW",
    level, -130.0, 30.0, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function configures the level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/This control sets the level mode.
/// HIPAR settingCharacteristic/This control sets the characteristic for the level setting.
/// HIPAR limit/This control sets the upper limit of the RF signal power.
ViStatus _VI_FUNC rssgu_ConfigureLevelSettings (ViSession instrSession,
                                                ViInt32 mode,
                                                ViInt32 settingCharacteristic,
                                                ViReal64 limit)
{
  ViStatus rssgu_status = VI_SUCCESS;

  CHECKERR (rssgu_SetLevelMode (instrSession, mode));

  rssgu_status = rssgu_SetLevelSettingCharacteristic (instrSession, settingCharacteristic);
  REMAP (2, 3);

  rssgu_status = rssgu_SetLevelLimit (instrSession, limit);
  REMAP (2, 4);

  return rssgu_status;
}

//=====================================================================================
/// HIFN This function configures frequency and level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR frequency/RF frequency.
/// HIPAR level/RF level.
ViStatus _VI_FUNC rssgu_SetFrequencyAndLevel (ViSession instrSession,
                                              ViReal64  frequency, ViReal64  level )
{
  ViUInt32 supported = 0;
  char buf[100] = {0};

  viFastSettingSupported (instrSession, IdSguFreq, &supported);

  if (supported)
  {

    ParaItem ParaList[2] = {
      {IdSguFreq, PARATYPE_REAL64, 0},
      {IdSguPow,  PARATYPE_REAL64, 0}};
      ParaList[0].pData = (unsigned char *)&frequency;
      ParaList[1].pData = (unsigned char *)&level;


      return viFastSetMultiple(instrSession, ParaList, 2);
  }

  sprintf (buf, "FREQ %f;POW %f", frequency, level);
  return rssgu_sendCmd(instrSession,buf);
}

//=====================================================================================
/// HIFN This function applies any signal generation related settings 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_ApplySettings (ViSession instrSession)
{

  ViUInt32 supported = 0;
  viFastSettingSupported (instrSession, IdSguApplySettings, &supported);
  if (supported)
    return viFastSetInt32(instrSession, IdSguApplySettings, 1);

  return rssgu_sendCmd(instrSession, "SOUR:SETT:APPL");
}

//=====================================================================================
/// HIFN This function sets the level mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/This control sets the level mode.
ViStatus _VI_FUNC rssgu_SetLevelMode (ViSession instrSession, 
                                      ViInt32 mode)
{
  return rssgu_setEnum (instrSession, 
    ":SOUR:POW:LMOD", 
    mode, 
    LevelModeArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the level mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/Displays the level mode.
ViStatus _VI_FUNC rssgu_GetLevelMode (ViSession instrSession, 
                                      ViInt32 *mode)
{
  return rssgu_getEnum (instrSession, 
    ":SOUR:POW:LMOD?", 
    mode, 
    LevelModeArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the characteristic for the level setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR settingCharacteristic/This control sets the characteristic for the level setting.
ViStatus _VI_FUNC rssgu_SetLevelSettingCharacteristic (ViSession instrSession,
                                                       ViInt32 settingCharacteristic)
{
  return rssgu_setEnum (instrSession, 
    ":SOUR:POW:SCH", 
    settingCharacteristic, 
    LevelSettingCharacteristicArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the characteristic for the level setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR settingCharacteristic/Displays the characteristic for the level setting.
ViStatus _VI_FUNC rssgu_GetLevelSettingCharacteristic (ViSession instrSession,
                                                       ViInt32 *settingCharacteristic)
{
  return rssgu_getEnum (instrSession, 
    ":SOUR:POW:SCH?", 
    settingCharacteristic, 
    LevelSettingCharacteristicArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the RF signal's power level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR level/Displays the RF signal's power level.
ViStatus _VI_FUNC rssgu_GetLevel (ViSession instrSession, ViReal64 *level)
{
  ViUInt32 supported = 0;
  viFastQuerySupported(instrSession, IdSguPow, &supported);
  if (supported)
    return viFastQueryReal64(instrSession, IdSguPow, level);

  return rssgu_getViReal64 (instrSession, 
    ":POW?", 
    level, 
    VI_ERROR_PARAMETER2);
}  

/// HIFN This function queries he RF signal's peak envelope power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR PEP/Returns he RF signal's peak envelope power.
ViStatus _VI_FUNC rssgu_GetPeakEnvelopePower (ViSession instrSession,
                                              ViReal64 *PEP)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:POW:PEP?", 
    PEP, 
    VI_ERROR_PARAMETER2);    
}

/// HIFN This function queries the minimum level range in the current level mode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR LowerLimit/Returns the minimum level range in the current level mode
ViStatus _VI_FUNC rssgu_GetLevelLowerRange(ViSession instrSession,
                                           ViReal64 *LowerLimit)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:POW:RANG:LOW?", 
    LowerLimit, 
    VI_ERROR_PARAMETER2);    
}

/// HIFN This function queries the maximum level range in the current level mode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR UpperLimit/Returns the maximum level range in the current level mode
ViStatus _VI_FUNC rssgu_GetLevelUpperRange(ViSession instrSession,
                                           ViReal64 *UpperLimit)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:POW:RANG:UPP?", 
    UpperLimit, 
    VI_ERROR_PARAMETER2);    
}

//=====================================================================================
/// HIFN This function sets the upper limit of the RF signal power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR limit/This control sets the upper limit of the RF signal power.
ViStatus _VI_FUNC rssgu_SetLevelLimit (ViSession instrSession,
                                       ViReal64 limit)
{
  return rssgu_setViReal64 (instrSession, 
    ":SOUR:POW:LIM:AMPL", 
    limit, -300.0, 30.0, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the upper limit of the RF signal power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR limit/Displays the upper limit of the RF signal power.
ViStatus _VI_FUNC rssgu_GetLevelLimit (ViSession instrSession,
                                       ViReal64 *limit)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:POW:LIM:AMPL?", 
    limit, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the state which the RF output assumes when the instrument is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR powerOnState/This control sets the state which the RF output assumes when the instrument is switched on.
ViStatus _VI_FUNC rssgu_SetLevelPowerOnState (ViSession instrSession,
                                              ViInt32 powerOnState)
{
  return rssgu_setEnum (instrSession, 
    ":OUTP:STAT:PON", 
    powerOnState, 
    LevelPowerOnStateArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the state which the RF output assumes when the instrument is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR powerOnState/Displays the state which the RF output assumes when the instrument is switched on.
ViStatus _VI_FUNC rssgu_GetLevelPowerOnState (ViSession instrSession,
                                              ViInt32 *powerOnState)
{
  return rssgu_getEnum (instrSession, 
    ":OUTP:STAT:PON?", 
    powerOnState, 
    LevelPowerOnStateArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the automatic level control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/This control sets automatic level control.
ViStatus _VI_FUNC rssgu_SetLevelALCState (ViSession instrSession,
                                          ViInt32 state)
{
  return rssgu_setEnum (instrSession, 
    ":SOUR:POW:ALC:STAT", 
    state, 
    LevelALCStateArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the automatic level control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/Displays the automatic level control.
ViStatus _VI_FUNC rssgu_GetLevelALCState (ViSession instrSession,
                                          ViInt32 *state)
{
  return rssgu_getEnum (instrSession, 
    ":SOUR:POW:ALC:STAT?", 
    state, 
    LevelALCStateArr, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function activate level control for correction purposes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_ReadjustLevelALC (ViSession instrSession)
{
  return rssgu_sendCmd (instrSession, ":SOUR:POW:ALC:SONC");
}

/// HIFN This function sets the state which the attenuator is to assume after the instrument is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/This control sets the state which the attenuator is to assume after the instrument is switched on.
ViStatus _VI_FUNC rssgu_SetAttenuatorModeAfterOn (ViSession instrSession,
                                                  ViInt32 mode)
{
  return rssgu_setEnum (instrSession, 
    ":SOUR:POW:ATT:RFOF:MODE", 
    mode, 
    attenuatorAfterArr, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the state which the attenuator is to assume after the instrument is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/Displays the state which the attenuator is to assume after the instrument is switched on.
ViStatus _VI_FUNC rssgu_GetAttenuatorModeAfterOn (ViSession instrSession,
                                                  ViInt32 *mode)
{
  return rssgu_getEnum (instrSession, 
    ":SOUR:POW:ATT:RFOF:MODE?", 
    mode, 
    attenuatorAfterArr, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function sets the switch-over offset value of the attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR offset/This control sets the switch-over offset value of the attenuator.
ViStatus _VI_FUNC rssgu_SetAttenuatorSwitchOverOffset (ViSession instrSession,
                                                       ViReal64 offset)
{
  return rssgu_setViReal64 (instrSession, 
    ":SOUR:POW:ATT:SOV", 
    offset, 
    -10.0, 10.0, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the switch-over offset value of the attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR offset/Displays the switch-over offset value of the attenuator.
ViStatus _VI_FUNC rssgu_GetAttenuatorSwitchOverOffset (ViSession instrSession,
                                                       ViReal64 *offset)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:POW:ATT:SOV?", 
    offset, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the minimum level which can be set without the attenuator being adjusted (Attenuator FIXed).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR attenuatorLowerRange/Returns the minimum level which can be set without the attenuator being adjusted (Attenuator FIXed).
ViStatus _VI_FUNC rssgu_GetAttenuatorLowerRange (ViSession instrSession,
                                                 ViReal64 *attenuatorLowerRange)
{
  return rssgu_getViReal64 (instrSession, 
    "OUTP:AFIX:RANG:LOW?", 
    attenuatorLowerRange, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the maximum level which can be set without the attenuator being adjusted (Attenuator FIXed).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR attenuatorUpperRange/Returns the maximum level which can be set without the attenuator being adjusted (Attenuator FIXed).
ViStatus _VI_FUNC rssgu_GetAttenuatorUpperRange (ViSession instrSession,
                                                 ViReal64 *attenuatorUpperRange)
{
  return rssgu_getViReal64 (instrSession, 
    "OUTP:AFIX:RANG:UPP?", 
    attenuatorUpperRange, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function configures the IQ Impairments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/This control sets the impairment or correction values LEAKage, QUADrature and IQRatio for the digital baseband signal.
/// HIPAR iOffset/This control sets the carrier offset for the I-channel of the I/Q modulation.
/// HIPAR qOffset/This control sets the carrier offset for the Q-channel of the I/Q modulation. 
/// HIPAR gainImbalance/This control sets the ratio of I modulation to Q modulation (gain imbalance).
/// HIPAR quadratureOffset/This control sets the quadrature offset for the digital I/Q signal. 
ViStatus _VI_FUNC rssgu_ConfigureIQImpairments (ViSession instrSession,
                                                ViBoolean state,
                                                ViReal64 iOffset,
                                                ViReal64 qOffset,
                                                ViReal64 gainImbalance,
                                                ViReal64 quadratureOffset)
{
  ViStatus rssgu_status = VI_SUCCESS;

  CHECKERR (rssgu_SetIQImpairmentsState (instrSession, state));
  REMAP (2, 2);

  rssgu_status = rssgu_SetIQImpairmentsIOffset (instrSession, iOffset);
  REMAP (2, 3);

  rssgu_status = rssgu_SetIQImpairmentsQOffset (instrSession, qOffset);
  REMAP (2, 4);

  rssgu_status = rssgu_SetIQImpairmentsGainImbalance (instrSession, gainImbalance);
  REMAP (2, 5);

  rssgu_status = rssgu_SetIQImpairmentsQuadratureOffset (instrSession, quadratureOffset);
  REMAP (2, 6);

  return rssgu_status;
}

//=====================================================================================
/// HIFN This function sets the state of the IQ modulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR IQModulatorState/This control sets the state of the IQ modulator.
ViStatus _VI_FUNC rssgu_SetIQModulatorState (ViSession instrSession,
                                             ViBoolean IQModulatorState)
{

  ViUInt32 supported = 0;
  viFastSettingSupported (instrSession, IdSguModState, &supported);
  if (supported)
    return viFastSetInt32(instrSession, IdSguModState, IQModulatorState);

  return rssgu_setViBoolean (instrSession, 
    ":SOUR:IQ:STAT", 
    IQModulatorState, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the state of the IQ modulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR IQModulatorState/Displays the state of the IQ modulator.
ViStatus _VI_FUNC rssgu_GetIQModulatorState (ViSession instrSession,
                                             ViBoolean *IQModulatorState)
{
  ViUInt32 supported = 0;
  viFastQuerySupported(instrSession, IdSguModState, &supported);
  if (supported)
  {
    ViInt32 sel = 0;
    int status = 0;
    status = viFastQueryInt32(instrSession, IdSguModState, &sel);
    *IQModulatorState = (sel != 0);
    return status;
  }

  return rssgu_getViBoolean (instrSession, 
    ":SOUR:IQ:STAT?", 
    IQModulatorState, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the impairment or correction values LEAKage, QUADrature and IQRatio for the digital baseband signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/This control sets the impairment or correction values LEAKage, QUADrature and IQRatio for the digital baseband signal.
ViStatus _VI_FUNC rssgu_SetIQImpairmentsState (ViSession instrSession,
                                               ViBoolean state)
{
  return rssgu_setViBoolean (instrSession, 
    ":SOUR:IQ:IMP:STAT", 
    state, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the impairment or correction values LEAKage, QUADrature and IQRatio for the digital baseband signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/Displays the impairment or correction values LEAKage, QUADrature and IQRatio for the digital baseband signal.
ViStatus _VI_FUNC rssgu_GetIQImpairmentsState (ViSession instrSession,
                                               ViBoolean *state)
{
  return rssgu_getViBoolean (instrSession, 
    ":SOUR:IQ:IMP:STAT?", 
    state, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the carrier offset for the I-channel of the I/Q modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR iOffset/This control sets the carrier offset for the I-channel of the I/Q modulation.
ViStatus _VI_FUNC rssgu_SetIQImpairmentsIOffset (ViSession instrSession,
                                                 ViReal64 iOffset)
{
  return rssgu_setViReal64 (instrSession, 
    ":SOUR:IQ:IMP:LEAK:I", 
    iOffset, -10.0, 10.0, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the carrier offset for the I-channel of the I/Q modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR iOffset/Displays the carrier offset for the I-channel of the I/Q modulation.
ViStatus _VI_FUNC rssgu_GetIQImpairmentsIOffset (ViSession instrSession,
                                                 ViReal64 *iOffset)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:IQ:IMP:LEAK:I?", 
    iOffset, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the carrier offset for the Q-channel of the I/Q modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR qOffset/This control sets the carrier offset for the Q-channel of the I/Q modulation.
ViStatus _VI_FUNC rssgu_SetIQImpairmentsQOffset (ViSession instrSession,
                                                 ViReal64 qOffset)
{
  return rssgu_setViReal64 (instrSession, 
    ":SOUR:IQ:IMP:LEAK:Q", 
    qOffset, -10.0, 10.0, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the carrier offset for the Q-channel of the I/Q modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR qOffset/Displays the carrier offset for the Q-channel of the I/Q modulation.

ViStatus _VI_FUNC rssgu_GetIQImpairmentsQOffset (ViSession instrSession,
                                                 ViReal64 *qOffset)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:IQ:IMP:LEAK:Q?", 
    qOffset,
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the ratio of I modulation to Q modulation (gain imbalance).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR gainImbalance/This control sets the ratio of I modulation to Q modulation (gain imbalance).
ViStatus _VI_FUNC rssgu_SetIQImpairmentsGainImbalance (ViSession instrSession,
                                                       ViReal64 gainImbalance)
{
  return rssgu_setViReal64 (instrSession, 
    ":SOUR:IQ:IMP:IQR", 
    gainImbalance, -1.000, 1.000, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the ratio of I modulation to Q modulation (gain imbalance).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR gainImbalance/Displays the ratio of I modulation to Q modulation (gain imbalance).
ViStatus _VI_FUNC rssgu_GetIQImpairmentsGainImbalance (ViSession instrSession,
                                                       ViReal64 *gainImbalance)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:IQ:IMP:IQR?", 
    gainImbalance, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function sets the quadrature offset for the digital I/Q signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR quadratureOffset/This control sets the quadrature offset for the digital I/Q signal.
ViStatus _VI_FUNC rssgu_SetIQImpairmentsQuadratureOffset (ViSession instrSession,
                                                          ViReal64 quadratureOffset)
{
  return rssgu_setViReal64 (instrSession, 
    ":SOUR:IQ:IMP:QUAD:ANGL", 
    quadratureOffset, -10.0, 10.0, 
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the quadrature offset for the digital I/Q signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR quadratureOffset/Displays the quadrature offset for the digital I/Q signal.
ViStatus _VI_FUNC rssgu_GetIQImpairmentsQuadratureOffset (ViSession instrSession,
                                                          ViReal64 *quadratureOffset)
{
  return rssgu_getViReal64 (instrSession, 
    ":SOUR:IQ:IMP:QUAD:ANGL?", 
    quadratureOffset,
    VI_ERROR_PARAMETER2);
}



/// HIFN This function sets the crest factor of the IQ modulation signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR crestFactor/Sets the crest factor of the IQ modulation signal.
ViStatus _VI_FUNC rssgu_SetIQCrestFactor (ViSession instrSession,
                                          ViReal64 crestFactor)
{
  ViUInt32 supported = 0;
  viFastSettingSupported (instrSession, IdSguCrestfact, &supported);
  if (supported)
    return viFastSetReal64(instrSession, IdSguCrestfact, crestFactor);

  return rssgu_setViReal64 (instrSession, 
    ":SOUR:IQ:CRES", 
    crestFactor, 0.0, 35.0, 
    VI_ERROR_PARAMETER2);    
}

/// HIFN This function queries the crest factor of the IQ modulation signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR crestFactor/Returns the crest factor of the IQ modulation signal.
ViStatus _VI_FUNC rssgu_GetIQCrestFactor (ViSession instrSession,
                                          ViReal64 *crestFactor)
{
  ViUInt32 supported = 0;
  viFastQuerySupported(instrSession, IdSguCrestfact, &supported);
  if (supported)
    return viFastQueryReal64(instrSession, IdSguCrestfact, crestFactor);

  return rssgu_getViReal64 (instrSession, 
    ":SOUR:IQ:CRES?", 
    crestFactor, 
    VI_ERROR_PARAMETER2);    
}    

//=====================================================================================
/// HIFN This function sets the state of the pulse modulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/This control sets the state of the pulse modulator.
ViStatus _VI_FUNC rssgu_SetPulseModulatorState (ViSession instrSession,
                                               ViBoolean state)
{
  return rssgu_setViBoolean (instrSession, 
    "PULM:STAT", 
    state,
    VI_ERROR_PARAMETER2);    
}

//=====================================================================================
/// HIFN This function queries the state of the pulse modulator
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/Displays the state of the signal output.
ViStatus _VI_FUNC rssgu_GetPulseModulatorState (ViSession instrSession,
                                               ViBoolean *state)
{
  return rssgu_getViBoolean (instrSession, 
    "PULM:STAT?", 
    state,
    VI_ERROR_PARAMETER2);    
}


/// HIFN This function sets the state which the attenuator is to assume after the instrument is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR polarity/This control sets the state which the attenuator is to assume after the instrument is switched on.
ViStatus _VI_FUNC rssgu_SetPulseModulatorPolarity (ViSession instrSession,
                                                  ViInt32 polarity)
{
  return rssgu_setEnum (instrSession, 
    ":PULM:POL", 
    polarity, 
    pulmPolArr, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function queries the state which the attenuator is to assume after the instrument is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR polarity/Displays the state which the attenuator is to assume after the instrument is switched on.
ViStatus _VI_FUNC rssgu_GetPulseModulatorPolarity (ViSession instrSession,
                                                  ViInt32 *polarity)
{
  return rssgu_getEnum (instrSession, 
    ":PULM:POL?", 
    polarity, 
    pulmPolArr, 
    VI_ERROR_PARAMETER2);
}


/// HIFN This function pass a lock request id which uniquely identifies the controller to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR lockID/This control uniquely identifies the controller to the instrument.
/// HIPAR result/Pass a lock request id which uniquely identifies the controller to the instrument.
ViStatus _VI_FUNC rssgu_LockInstrument (ViSession instrSession, ViInt32 lockID,
                                        ViInt32 *result)
{
  ViChar buffer[BUFFER_SIZE];

  sprintf (buffer, "LOCK? %ld", lockID);

  return rssgu_getViInt32 (instrSession, 
    buffer,
    result,
    VI_ERROR_PARAMETER3);
}

/// HIFN This function unlock device.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR lockID/This control uniquely identifies the controller to the instrument.
ViStatus _VI_FUNC rssgu_UnlockInstrument (ViSession instrSession,
                                          ViInt32 lockID)
{
  ViChar buffer[BUFFER_SIZE];

  sprintf (buffer, "UNL %ld", lockID);

  return rssgu_sendCmd (instrSession, buffer);
}

//=====================================================================================
/// HIFN This function performs all adjustments which affect the synthesis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_CALibrationLevelStart (ViSession instrSession)
{
  return rssgu_sendCmd (instrSession, "CAL:LEV:START");    
}

//=====================================================================================
/// HIFN This function performs all adjustments which affect the synthesis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR result/Returns result of the adjustment.
ViStatus _VI_FUNC rssgu_CALibrationLevelResult (ViSession instrSession,
                                                ViInt32 *result)
{
  return rssgu_getEnum (instrSession, 
    "CAL:LEV:RES?", 
    result, 
    testResult, 
    VI_ERROR_PARAMETER2);
}

/// HIFN This function starts the adjustment for the I/Q modulator for the entire frequency range. The I/Q modulator is adjusted with respect to carrier leakage, I/Q imbalance and quadrature.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_CALibrationIQModulatorStart (ViSession instrSession)
                                                
{
  return rssgu_sendCmd (instrSession, "CAL:IQM:START");    
}

/// HIFN Queries the result of the internal I/Q adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR result/Displays the result of the internal I/Q adjustment.
ViStatus _VI_FUNC rssgu_CALibrationIQModulatorResult (ViSession instrSession,
                                                ViInt32 *result)
{
  return rssgu_getEnum (instrSession, 
    "CAL:IQM:RES?", 
    result, 
    testResult, 
    VI_ERROR_PARAMETER2);
}


/// HIFN This function starts the adjustment for the I/Q modulator for the current frequency. The I/Q modulator is adjusted with respect to carrier leakage, I/Q imbalance and quadrature.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_CALibrationIQModulatorForCurrentFrequencyStart
(ViSession instrSession)
{
  return rssgu_sendCmd (instrSession, "CAL:IQM:LOC:START");    

}


/// HIFN Queries the result of the I/Q modulation adjustment for the current frequency. 
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR result/Returns the result of the I/Q modulation adjustment for the current frequency. 
ViStatus _VI_FUNC rssgu_CALibrationIQModulatorForCurrentFrequencyResult (ViSession instrSession,
                                                                         ViInt32 *result)
{
  return rssgu_getEnum (instrSession, 
    "CAL:IQM:LOC:RES?", 
    result, 
    testResult, 
    VI_ERROR_PARAMETER2);
}


/// HIFN Query calibration or selftest command from device. The caller must interpret the reply and forward it to a signal generator or interpret it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR command/Returns command from device
ViStatus _VI_FUNC rssgu_CALibrationGetHostRequest (ViSession instrSession,
                                                ViInt32 arraySize,
                                                ViChar _VI_FAR command[])
{
  return rssgu_getViStringQuoted (instrSession, 
    ":CALibration:HREQuest?", 
    "\"",
    arraySize,
    command, 
    VI_ERROR_PARAMETER3);
}                                                

//=====================================================================================
/// HIFN This function queries the delta temperature since the last performed IQ adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR temperature/Displays the delta temperature since the last performed IQ adjustment
ViStatus _VI_FUNC rssgu_GetCalibrationIQTemperature (ViSession instrSession,
                                                     ViReal64 *temperature)
{
  return rssgu_getViReal64 (instrSession,
    ":CAL:IQM:TEMP?",
    temperature,
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the delta temperature since the last performed level adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR temperature/Displays the delta temperature since the last performed level adjustment
ViStatus _VI_FUNC rssgu_GetCalibrationLevelTemperature (ViSession instrSession,
                                                        ViReal64 *temperature)
{
  return rssgu_getViReal64 (instrSession,
    ":CAL:LEV:TEMP?",
    temperature,
    VI_ERROR_PARAMETER2);
}

/// HIFN This function reads out the number of traits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR count/Reads back number of traits.
ViStatus _VI_FUNC rssgu_GetTraitsCount (ViSession instrSession,
                                        ViInt32* count)
{
    return rssgu_getViInt32(instrSession,
      "TRA:COUN?",
      count,
      VI_ERROR_PARAMETER3);
}

/// HIFN This function queries the list of traits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR propertyIndex/This control selects the property for which to get its traits
/// HIPAR arraySize/This control sets the array size.
/// HIPAR traits/List of traits
ViStatus _VI_FUNC rssgu_GetTraits (ViSession instrSession,
                                                   ViInt32 propertyIndex,
                                                   ViInt32 arraySize,
                                                   ViChar _VI_FAR traits[])
{
    char cmd[100];
    if (propertyIndex < 0)
    {
      return VI_ERROR_PARAMETER2;
    }
    sprintf (cmd,":TRA%d?", propertyIndex+1);
    return rssgu_getCatalog (instrSession,
      cmd,
      (ViUInt32) arraySize,
      traits,
      VI_ERROR_PARAMETER3);
}  

//=====================================================================================
/// HIFN This function queries the device's operating system name
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the array size.
/// HIPAR hostname/Displays the device's operating system name
ViStatus _VI_FUNC rssgu_GetSYSTemOperatingSystem (ViSession instrSession,
                                            ViInt32 arraySize,
                                            ViChar hostname[])
{
  return rssgu_getViStringQuoted (instrSession, 
    ":SYST:OSYS?", 
    "\"",
    arraySize,
    hostname, 
    VI_ERROR_PARAMETER3);
}

//=====================================================================================
/// HIFN This function queries the path to the user accessible memory
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the array size.
/// HIPAR path/Displays path to the user memory
ViStatus _VI_FUNC rssgu_GetSYSTemUserMemoryPath (ViSession instrSession,
                                                 ViInt32 arraySize,
                                                 ViChar path[])
{
  return rssgu_getViStringQuoted (instrSession, 
    ":SYST:MMEM:PATH:USER?", 
    "\"",
    arraySize,
    path, 
    VI_ERROR_PARAMETER3);
}

//=====================================================================================
/// HIFN This function gets network settings from system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_GetNetworkSystemSettings (ViSession instrSession)
{
  return rssgu_sendCmd (instrSession, 
    ":SYST:COMM:NETW:GET");
}

//=====================================================================================
/// HIFN This function queries the MAC Address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the array size.
/// HIPAR MACAddress/Displays the MAC Address.
ViStatus _VI_FUNC rssgu_GetNetworkMACAddress (ViSession instrSession,
                                              ViInt32 arraySize,
                                              ViChar MACAddress[])
{
  return rssgu_getViStringQuoted (instrSession, 
    ":SYST:COMM:NETW:MAC?", 
    "\"",
    arraySize, 
    MACAddress, 
    VI_ERROR_PARAMETER3);
}

//=====================================================================================
/// HIFN This function configures the IP Address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR addressMode/This control sets the IP Address Mode.
/// HIPAR IPAddress/This control sets the IP Address.
/// HIPAR subnetMask/This control sets the Subnet Mask.
/// HIPAR defaultGateway/This control sets the Default Gateway.
ViStatus _VI_FUNC rssgu_ConfigureNetworkIP (ViSession instrSession,
                                            ViInt32 addressMode,
                                            ViString IPAddress,
                                            ViString subnetMask,
                                            ViString defaultGateway)
{
  ViStatus rssgu_status = VI_SUCCESS;

  CHECKERR (rssgu_SetNetworkIPAddressMode (instrSession, addressMode));

  rssgu_status = rssgu_SetNetworkIPAddress (instrSession, IPAddress);
  REMAP (2, 3);

  rssgu_status = rssgu_SetNetworkIPSubnetMask (instrSession, subnetMask);
  REMAP (2, 4);

  rssgu_status = rssgu_SetNetworkIPDefaultGateway (instrSession, defaultGateway);
  REMAP (2, 5);

  return rssgu_status;
}

/// HIFN This function restarts the network interface
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rssgu_NetworkRestart (ViSession instrSession)
{
  return rssgu_sendCmd (instrSession, "SYST:COMM:NETW:REST");
}

/// HIFN This function returns the resource string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR arraySize/Array size of the parameter labeled 'Resource'.
/// HIPAR resource/This control displays the resource string.
ViStatus _VI_FUNC rssgu_GetNetworkResource(
  ViSession   instrSession,
  ViUInt32 arraySize,
  ViChar _VI_FAR  resource[]
  )
  {
    return rssgu_getViStringQuoted (instrSession,
      "SYST:COMM:NETW:RES?",
      "\"",
      arraySize,
      resource,
      VI_ERROR_PARAMETER3);
  }

  /// HIFN This function returns the network configuration state.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR configuration/This control displays the network configuration state.
  ViStatus _VI_FUNC rssgu_GetNetworkConfigurationState (ViSession instrSession,
    ViBoolean *configuration)
  {
    return rssgu_getViBoolean (instrSession,
      "SYST:COMM:NETW:STAT?",
      configuration,
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function sets the Hostname of the network.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR hostname/This control sets the Hostname of the network.
  ViStatus _VI_FUNC rssgu_SetNetworkCommonHostname (ViSession instrSession,
    ViString hostname)
  {
    return rssgu_setViStringQuoted (instrSession, 
      ":SYST:COMM:NETW:COMM:HOST", 
      "'",
      hostname, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function queries the Hostname of the network.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR hostname/Displays the Hostname of the network.
  ViStatus _VI_FUNC rssgu_GetNetworkCommonHostname (ViSession instrSession,
    ViInt32 arraySize,
    ViChar hostname[])
  {
    return rssgu_getViStringQuoted (instrSession, 
      ":SYST:COMM:NETW:COMM:HOST?", 
      "\"",
      arraySize,
      hostname, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function sets the IP Address Mode.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR addressMode/This control sets the IP Address Mode.
  ViStatus _VI_FUNC rssgu_SetNetworkIPAddressMode (ViSession instrSession,
    ViInt32 addressMode)
  {
    return rssgu_setEnum (instrSession, 
      "SYST:COMM:NETW:IPAD:MODE", 
      addressMode,
      ConfigureNetworkIPArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function queries the IP Address Mode.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR addressMode/Displays the IP Address Mode.
  ViStatus _VI_FUNC rssgu_GetNetworkIPAddressMode (ViSession instrSession,
    ViInt32 *addressMode)
  {
    return rssgu_getEnum (instrSession, 
      "SYST:COMM:NETW:IPAD:MODE?", 
      addressMode,
      ConfigureNetworkIPArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function sets the IP Adrress.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR IPAddress/This control sets the IP Address.
  ViStatus _VI_FUNC rssgu_SetNetworkIPAddress (ViSession instrSession,
    ViString IPAddress)
  {
    return rssgu_setViStringQuoted (instrSession,
      ":SYST:COMM:NETW:IPAD",
      "'",
      IPAddress,
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function queries the IP Adrress.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR IPAddress/Displays the IP Address.
  ViStatus _VI_FUNC rssgu_GetNetworkIPAddress (ViSession instrSession,
    ViInt32 arraySize,
    ViChar IPAddress[])
  {
    return rssgu_getViString (instrSession,
      ":SYST:COMM:NETW:IPAD?",
      IPAddress,
      arraySize,
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function sets the Subnet Mask.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR subnetMask/This control sets the Subnet Mask.
  ViStatus _VI_FUNC rssgu_SetNetworkIPSubnetMask (ViSession instrSession,
    ViString subnetMask)
  {
    return rssgu_setViStringQuoted (instrSession, 
      ":SYST:COMM:NETW:IPAD:SUBN:MASK", 
      "'",
      subnetMask, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function queries the Subnet Mask.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR subnetMask/Displays the Subnet Mask.
  ViStatus _VI_FUNC rssgu_GetNetworkIPSubnetMask (ViSession instrSession,
    ViInt32 arraySize,
    ViChar subnetMask[])
  {
    return rssgu_getViString (instrSession, 
      ":SYST:COMM:NETW:IPAD:SUBN:MASK?", 
      subnetMask, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function sets the Default Gateway.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR defaultGateway/This control sets the Default Gateway.
  ViStatus _VI_FUNC rssgu_SetNetworkIPDefaultGateway (ViSession instrSession,
    ViString defaultGateway)
  {
    return rssgu_setViStringQuoted (instrSession, 
      ":SYST:COMM:NETW:IPAD:GAT", 
      "'",
      defaultGateway, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function queries the Default Gateway.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR defaultGateway/Displays the Default Gateway.
  ViStatus _VI_FUNC rssgu_GetNetworkIPDefaultGateway (ViSession instrSession,
    ViInt32 arraySize,
    ViChar defaultGateway[])
  {
    return rssgu_getViString (instrSession, 
      ":SYST:COMM:NETW:IPAD:GAT?", 
      defaultGateway, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function accepts the IP Address Settings.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_AcceptNetworkIPSettings (ViSession instrSession)
  {
    // leave for compatibility
    return VI_SUCCESS;
  }

  
  //=====================================================================================
  /// HIFN This function sets the baud rate of the serial port.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR baudrate/Displays the baud rate

  ViStatus _VI_FUNC rssgu_SetSerialBaudRate (ViSession instrSession,
                                             ViInt32 baudrate)
  {
    return rssgu_setEnum (instrSession, 
      "SYST:COMM:SER:BAUD", 
      baudrate,
      SerialBaudRateArr, 
      VI_ERROR_PARAMETER2);

  }


  //=====================================================================================
  /// HIFN This function queries the baud rate of the serial port.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR baudrate/Displays the baud rate

  ViStatus _VI_FUNC rssgu_GetSerialBaudRate (ViSession instrSession,
                                             ViInt32* baudrate)
  {
    return rssgu_getEnum (instrSession, 
      "SYST:COMM:SER:BAUD?", 
      baudrate,
      SerialBaudRateArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function sets the parity setting of the serial port.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR parity/Displays the parity setting

  ViStatus _VI_FUNC rssgu_SetSerialParity (ViSession instrSession,
                                           ViInt32 parity)
  {
    return rssgu_setEnum (instrSession, 
      "SYST:COMM:SER:PAR", 
      parity,
      SerialParityArr, 
      VI_ERROR_PARAMETER2);

  }


  //=====================================================================================
  /// HIFN This function queries the parity setting of the serial port.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR parity/Displays the parity setting

  ViStatus _VI_FUNC rssgu_GetSerialParity (ViSession instrSession,
                                             ViInt32* parity)
  {
    return rssgu_getEnum (instrSession, 
      "SYST:COMM:SER:PAR?", 
      parity,
      SerialParityArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function sets the stop bits setting of the serial port.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR stopbits/Displays the stop bits setting

  ViStatus _VI_FUNC rssgu_SetSerialStopBits (ViSession instrSession,
    ViInt32 stopbits)
  {
    return rssgu_setEnum (instrSession, 
      "SYST:COMM:SER:SBIT", 
      stopbits,
      SerialStopBitsArr, 
      VI_ERROR_PARAMETER2);

  }


  //=====================================================================================
  /// HIFN This function queries the stop bits setting of the serial port.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR stopbits/Displays the stop bits setting

  ViStatus _VI_FUNC rssgu_GetSerialStopBits (ViSession instrSession,
    ViInt32* stopbits)
  {
    return rssgu_getEnum (instrSession, 
      "SYST:COMM:SER:SBIT?", 
      stopbits,
      SerialStopBitsArr, 
      VI_ERROR_PARAMETER2);
  }



  //=====================================================================================
  /// HIFN This function sets the remote control operation mode.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR operationMode/This control sets the remote control operation mode.
  ViStatus _VI_FUNC rssgu_SetRemoteOperationMode (ViSession instrSession,
    ViInt32 operationMode)
  {
    return rssgu_setEnum (instrSession, 
      "REM:OPM", 
      operationMode,
      RemoteOperationModeArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function queries the remote control operation mode.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR operationMode/Displays the remote control operation mode.
  ViStatus _VI_FUNC rssgu_GetRemoteOperationMode (ViSession instrSession,
    ViInt32 *operationMode)
  {
    return rssgu_getEnum (instrSession, 
      "REM:OPM?", 
      operationMode,
      RemoteOperationModeArr, 
      VI_ERROR_PARAMETER2);
  } 


  //=====================================================================================
  /// HIFN CLEAR STATUS sets the status byte (STB), the standard event register (ESR), and the EVENt part of the QUEStionable and OPERation register to zero.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_ClearStatus (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, 
      "*CLS");    
  }

  //=====================================================================================
  /// HIFN EVENT STATUS ENABLE sets the event status enable register to the specified value.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/EVENT STATUS ENABLE sets the event status enable register to the specified value.
  ViStatus _VI_FUNC rssgu_EventStatusEnable (ViSession instrSession,
    ViInt32 registerValue)
  {
    return rssgu_setViInt32 (instrSession, 
      "*ESE", 
      registerValue, 0, 255, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN STANDARD EVENT STATUS QUERY returns the content of the event status register in decimal form (0...255) and then sets the register to zero.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Returns the content of the event status register in decimal form (0...255).
  ViStatus _VI_FUNC rssgu_EventStatusQuery (ViSession instrSession,
    ViInt32 *registerValue)
  {
    return rssgu_getViInt32 (instrSession, 
      "*ESR?", 
      registerValue, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN IDENTIFICATION QUERY queries the instrument identification. The instrument type, serial number and firmware version are returned.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR instrumentIdentification/Returns the instrument identification string.
  ViStatus _VI_FUNC rssgu_IdentificationQuery (ViSession instrSession,
    ViChar instrumentIdentification[])
  {
    return rssgu_getViString (instrSession, 
      "*IDN?", 
      instrumentIdentification, 
      256, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN INDIVIDUAL STATUS QUERY returns the content of the IST flag in decimal form (0 | 1).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR ISTFlag/Returns the content of the IST flag in decimal form (0 | 1).
  ViStatus _VI_FUNC rssgu_IndividualStatusQuery (ViSession instrSession,
    ViInt32 *ISTFlag)
  {
    return rssgu_getViInt32 (instrSession, 
      "*IST?", 
      ISTFlag, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN OPERATION COMPLETE sets bit 0 in the event status register when all previous commands have been processed.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_OperationComplete (ViSession instrSession)
  {
    ViStatus            rssgu_status;
    rssgu_instrRange    instrPtr = NULL;
    ViBoolean           err_check;
    unsigned int        ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    if (instrPtr == NULL)
      return VI_ERROR_PARAMETER1;

    err_check = instrPtr -> errorChecking;

    CHECKERR (rssgu_errorCheckState (instrSession, VI_FALSE));

    rssgu_status = rssgu_sendCmd (instrSession, "*OPC");
    (void) rssgu_errorCheckState (instrSession, err_check);

    return rssgu_status;
  }

  //=====================================================================================
  /// HIFN OPERATION COMPLETE QUERY returns a 1 when all previous commands have been processed.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR OPCState/Returns a 1 when all previous commands have been processed.
  ViStatus _VI_FUNC rssgu_OperationCompleteQuery (ViSession instrSession,
    ViInt32 *OPCState)
  {
    return rssgu_getViInt32 (instrSession, 
      "*OPC?", 
      OPCState, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN OPTION IDENTIFICATION QUERY queries the options configured in the instrument and returns a list of the installed options.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR optionIdentification/Returns a list of the installed options.
  ViStatus _VI_FUNC rssgu_OptionIdentificationQuery (ViSession instrSession,
    ViChar optionIdentification[])
  {
    return rssgu_getViString (instrSession, 
      "*OPT?",
      optionIdentification, 
      1024, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN PARALLEL POLL REGISTER ENABLE sets the parallel poll enable register to the specified value.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Sets the parallel poll enable register to the specified value.
  ViStatus _VI_FUNC rssgu_ParallelPollRegisterEnable (ViSession instrSession,
    ViInt32 registerValue)
  {
    return rssgu_setViInt32 (instrSession, 
      "*PRE", 
      registerValue, 0, 655, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN PARALLEL POLL REGISTER QUERY returns the content of the parallel poll enable register in decimal form.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Returns the content of the parallel poll enable register in decimal form.
  ViStatus _VI_FUNC rssgu_ParallelPollRegisterQuery (ViSession instrSession,
    ViInt32 *registerValue)
  {
    return rssgu_getViInt32 (instrSession, 
      "*PRE?", 
      registerValue, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN POWER ON STATUS CLEAR determines whether the content of the ENABle registers are retained or reset at power on.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Determines whether the content of the ENABle registers are retained or reset at power on.
  ViStatus _VI_FUNC rssgu_PowerOnStatusClear (ViSession instrSession,
    ViInt32 registerValue)
  {
    return rssgu_setViInt32 (instrSession, 
      "*PSC", 
      registerValue, 0, 1, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN POWER ON STATUS CLEAR QUERY retrieves the contents of the Power-on-Status-Clear flag.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Returns the contents of the Power-on-Status-Clear flag. The answer is either 0 or 1.
  ViStatus _VI_FUNC rssgu_PowerOnStatusClearQuery (ViSession instrSession,
    ViInt32 *registerValue)
  {
    return rssgu_getViInt32 (instrSession, 
      "*PSC?", 
      registerValue,
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN RECALL calls up the instrument status which was stored under the specified number using the *SAV command or using the MMEMory:STORe:STATe command. "0" corresponds to the power-on state, unless a new instrument setting was stored under this number in the meantime using the MMEMory:STORe:STATe command.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR instrumentStatus/Number under which is stored instrument setting.
  ViStatus _VI_FUNC rssgu_Recall (ViSession instrSession,
    ViInt32 instrumentStatus)
  {
    return rssgu_setViInt32 (instrSession, 
      "*RCL", 
      instrumentStatus, 0, 10, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN SAVE stores the current device state under the specified number (see also *RCL).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR instrumentStatus/Number under which will be stored instrument setting.
  ViStatus _VI_FUNC rssgu_Save (ViSession instrSession,
    ViInt32 instrumentStatus)
  {
    return rssgu_setViInt32 (instrSession, 
      "*SAV", 
      instrumentStatus, 0, 10, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN SERVICE REQUEST ENABLE sets the service request enable register to the specified value.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Ssets the service request enable register to the specified value.
  ViStatus _VI_FUNC rssgu_ServiceRequestEnable (ViSession instrSession,
    ViInt32 registerValue)
  {
    return rssgu_setViInt32 (instrSession, 
      "*SRE", 
      registerValue, 0, 255, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN SERVICE REQUEST ENABLE reads out the content of the service request enable register in decimal form.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerValue/Returns the content of the service request enable register in decimal form.
  ViStatus _VI_FUNC rssgu_ServiceRequestEnableQuery (ViSession instrSession,
    ViInt32 *registerValue)
  {
    return rssgu_getViInt32 (instrSession, 
      "*SRE?", 
      registerValue, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN READ STATUS BYTE QUERY reads out the content of the status byte in decimal form.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR statusByte/Returns the content of the status byte in decimal form.
  ViStatus _VI_FUNC rssgu_StatusByteQuery (ViSession instrSession,
    ViInt32 *statusByte)
  {
    return rssgu_getViInt32 (instrSession, 
      "*STB?", 
      statusByte, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN TRIGGER triggers all actions which are waiting for a trigger event.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_Trigger (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, 
      "*TRG");
  }

  //=====================================================================================
  /// HIFN This function triggers all actions waiting for a trigger event in the currently active window and waits for operation completed (OPC) before returning the status code.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR timeout/Sets the timeout for the triggering routine to be finished.
  ViStatus _VI_FUNC rssgu_TriggerWaitOPC (ViSession instrSession,
    ViInt32 timeout)
  {
    ViStatus rssgu_status = VI_SUCCESS;
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, timeout, 0, 600000), 2);

    CHECKERR (rssgu_waitOPC (instrSession, "*TRG;*OPC?", buffer, timeout));

    return rssgu_status;
  } 

//=====================================================================================
/// HIFN This function sets the operation mode of the trigger connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/This control sets the operation mode of the trigger connector.
ViStatus _VI_FUNC rssgu_SetTrigConnnectorOpMode (ViSession instrSession, 
                                                ViInt32 mode)
{
  return rssgu_setEnum (instrSession, 
    ":CONNector:TRIGger:OMODe", 
    mode,   
    trigConOpModeArr,    
    VI_ERROR_PARAMETER2);
}

//=====================================================================================
/// HIFN This function queries the operation mode of the trigger connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR mode/Displays the operation mode of the trigger connector.
ViStatus _VI_FUNC rssgu_GetTrigConnnectorOpMode (ViSession instrSession, 
                                                 ViPInt32 mode)
{
  return rssgu_getEnum (instrSession, 
    ":CONNector:TRIGger:OMODe?", 
    mode,   
    trigConOpModeArr,    
    VI_ERROR_PARAMETER2);

}

  //=====================================================================================
  /// HIFN SELF TEST QUERY triggers all self tests of the instrument and outputs an error code in decimal form (see Service Manual supplied with the instrument).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR errorCode/Returns an error code of the self tests.
  ViStatus _VI_FUNC rssgu_SelfTestQuery (ViSession instrSession,
    ViInt32 *errorCode)
  {
    return rssgu_getViInt32 (instrSession, 
      "*TST?", 
      errorCode, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN WAIT-to-CONTINUE does not allow subsequent commands to be processed until all previous commands have been executed and all signals are in their transient condition.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_WaittoContinue (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, 
      "*WAI");
  }



  /// HIFN This function queries the test points available in the instrument. A detailed description of the test points can be found in Chapter 3 of the Service Manual (supplied with the instrument).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR testPoints/Returns the test points available in the instrument (the various entries are separated by commas).
  ViStatus _VI_FUNC rssgu_DIAGnosticTestPoints (ViSession instrSession, 
    ViInt32 arraySize, 
    ViChar _VI_FAR testPoints[])
  {
    ViChar  buffer [BUFFER_SIZE];

    sprintf (buffer, "DIAG:POIN:CAT?");

    return rssgu_getCatalog (instrSession,
      buffer,
      (ViUInt32) arraySize,
      testPoints,
      VI_ERROR_PARAMETER3);
  }

  /// HIFN This function triggers voltage measurement at the specified test point and returns the measured voltage. A detailed description of the test points can be found in Chapter 3 of the Service Manual (supplied with the instrument). A list of the available test points can be queried using the DIAG:POIN:CAT? command .
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR testPointName/Test point name.
  /// HIPAR measuredVoltage/Returns the voltage measured at the specified test point.
  ViStatus _VI_FUNC rssgu_DIAGnosticTestPointVoltage (ViSession instrSession,
    ViString testPointName,
    ViReal64 *measuredVoltage)
  {
    ViChar      buffer [BUFFER_SIZE];

    CHECKNULL (testPointName, 2);

    sprintf (buffer, "DIAG:POIN? '%s'", testPointName);

    return rssgu_getViReal64 (instrSession,
      buffer,
      measuredVoltage,
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function returns the contents of a register on a specific device. For service purposes only.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR bar/identifies the device
  /// HIPAR address/the register's address   
  /// HIPAR data/the returned register contents 
  ViStatus _VI_FUNC rssgu_DIAGnosticGetMemoryContent (ViSession instrSession,
    ViInt32 bar,
    ViInt32 address,
    ViPInt32 data)
  {
    ViStatus rssgu_status = VI_SUCCESS;
    *data = 0;

    CHECKERR (rsPrintf (instrSession,":TEST:DIR? \"%s\",%d\n", mmacBarArr[bar],address));
    CHECKERR (rsScanf (instrSession, "#H%x", data));

    CHECKERR( rssgu_checkStatus (instrSession));


    return rssgu_status;

  }
  
  //=====================================================================================
  /// HIFN This function sets the contents of a register on a specific device. For service purposes only.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR bar/identifies the device
  /// HIPAR address/the register's address   
  /// HIPAR data/the register contents
  ViStatus _VI_FUNC rssgu_DIAGnosticSetMemoryContent (ViSession instrSession,
    ViInt32 bar,
    ViInt32 address,
    ViInt32 data)
  {
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR (rsPrintf (instrSession,":TEST:DIR \"%s\",%d,#H%08x\n", mmacBarArr[bar],address, data));
    CHECKERR( rssgu_checkStatus (instrSession));

    return rssgu_status;
  }                                                    

  //=====================================================================================
  /// HIFN This function retrieves information about the device's memory usage
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR TotalPhysMemKb/Total system memory 
  /// HIPAR ApplicMemKb/Application memory
  /// HIPAR HeapUsedKb/Used heap memory
  /// HIPAR HeapAvailableKb/Available heap memory

  ViStatus _VI_FUNC rssgu_MemoryHeapFree (ViSession instrSession,
                                          ViInt32* TotalPhysMemKb,
                                          ViInt32* ApplicMemKb,
                                          ViInt32* HeapUsedKb,
                                          ViInt32* HeapAvailableKb)
  {
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR (rsPrintf (instrSession,":MEM:HFR?\n"));
    CHECKERR (rsScanf (instrSession, "%d,%d,%d,%d",TotalPhysMemKb,ApplicMemKb,HeapUsedKb,HeapAvailableKb));

    CHECKERR( rssgu_checkStatus (instrSession));   
    return rssgu_status;
  }


  /// HIFN This function changes the default drive. This drive is used for all subsequent MMEM commands if no path is specified with them.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR drive/Drive name parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryChangeDefaultDrive (ViSession instrSession,
    ViString drive)
  {
    return rssgu_setViStringQuoted (instrSession,
      "MMEM:MSIS",
      "'",
      drive,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function queries the default drive. This drive is used for all subsequent MMEM commands if no path is specified with them.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR drive/Default Drive name.
  /// HIPAR arraySize/Size of drive array.
  ViStatus _VI_FUNC rssgu_MMEMoryGetDefaultDrive (ViSession instrSession,
    ViInt32 arraySize,
    ViChar drive[])
  {
    return rssgu_getViStringQuoted (instrSession,
      "MMEM:MSIS?",
      "\"",
      arraySize,
      drive,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function changes the default directory. This directory is used for all subsequent MMEM commands if no path is specified with them. It is also possible to change to a higher directory using two dots '..'.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryName/Directory name parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryChangeDirectory (ViSession instrSession,
    ViString directoryName)
  {
    return rssgu_setViStringQuoted (instrSession,
      "MMEM:CDIR",
      "'",
      directoryName,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function queries the current working directory.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/Defines size of array in argument Directory..
  /// HIPAR directory/Directory name parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryGetDirectory (ViSession instrSession,
    ViInt32 arraySize,
    ViChar directory[])
  {
    return rssgu_getViStringQuoted (instrSession,
      "MMEM:CDIR?",
      "\"",
      arraySize,
      directory,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function copies the first specified file to the second specified file. Instead of just a file, this function can also be used to copy a complete directory together with all its files.
  /// HIFN If <destination> is not specified, <source> is copied to the MMEM:MSIS drive and the MMEM:CDIR directory. Files which already exist with the same name in the destination directory are overwritten without an error message.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR sourcePath/Source path of the file or a complete directory together with all its files.
  /// HIPAR destinationPath/Destination path of the file or a directory. If 'Destination Path' is not specified, 'Source Path' is copied to the MMEM:MSIS drive and the MMEM:CDIR directory.
  ViStatus _VI_FUNC rssgu_MMEMoryCopy (ViSession instrSession, ViString sourcePath,
    ViString destinationPath)
  {
    ViStatus    rssgu_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKNULL (sourcePath, 2);

    if (destinationPath)
      sprintf (buffer, "MMEM:COPY '%s','%s'", sourcePath, destinationPath);
    else
      sprintf (buffer, "MMEM:COPY '%s'", sourcePath);

    rssgu_status = rssgu_sendCmd (instrSession, buffer);

    return rssgu_status;
  }

  /// HIFN This function writes the block data <binary block data> to the file identified by <file_name>. The IEC/IEEE-bus terminator should be set to EOI in order to ensure correct data transfer.
  /// HIFN This function can be used to transfer stored instrument settings or waveforms directly to the instrument.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR fileName/File name of the binary block data.
  /// HIPAR binaryBlockLength/Defines the number of bytes in the 'Binary Block Data' array.
  /// HIPAR binaryBlockData/Binary block data array to be writen to the file defined by  'File Name' parameter with the size limited with 'Binary Block Length' parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryBlockDataWrite (ViSession instrSession,
    ViString fileName,
    ViInt32 binaryBlockLength,
    ViChar _VI_FAR binaryBlockData[])
  {
    ViStatus    rssgu_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViChar*     pbuffer;
    ViChar*     p2buf;

    CHECKNULL (fileName, 2);
    CHECKNULL (binaryBlockData, 4);

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
      CHECKERR (rsWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    p2buf = pbuffer = (ViChar *)malloc (strlen (fileName) + binaryBlockLength + 1000);

    p2buf += sprintf (pbuffer, "MMEM:DATA '%s',#%ld%ld",
      fileName,
      (ViInt32)(floor(log10((double)binaryBlockLength)) + 1),
      binaryBlockLength);

    memcpy (p2buf, binaryBlockData, (size_t) binaryBlockLength);
    p2buf[binaryBlockLength] = '\n';

    rssgu_status = rsWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) ((p2buf - pbuffer) + binaryBlockLength + 1), VI_NULL);

    if (pbuffer)
      free (pbuffer);

    if (intfType == VI_INTF_GPIB)
      CHECKERR (rsWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL));

    return rssgu_status;
  }

  /// HIFN This function transfers the specified file from the R&S SGU to the IEC/IEEE bus and then on to the control computer. It is important to ensure that the intermediate memory on the control computer is large enough to take the file. In this case, the setting for the IEC/IEEE-bus terminator is irrelevant.
  /// HIFN This function can be used to read stored instrument settings or waveforms directly from the instrument.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR fileName/File name of the binary block data.
  /// HIPAR binaryBlockLength/Length of the binary block data array.
  /// HIPAR binaryBlockData/Binary block data to be retrieved from the file defined by the 'File Name' parameter. Array size of this parameter must be large enough to store all the retrieved data.
  ViStatus _VI_FUNC rssgu_MMEMoryBlockDataRead (ViSession instrSession,
    ViString fileName,
    ViInt32 binaryBlockLength,
    ViChar _VI_FAR binaryBlockData[])
  {
    ViStatus    rssgu_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
      lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
      format[32],
      *pbuffer = NULL, *p2buffer = NULL;

    CHECKNULL (fileName, 2);

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
      CHECKERR (rsWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "MMEM:DATA? '%s'\n", fileName);

    CHECKERR (rsWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssgu_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
    {
      free (pbuffer);
      return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
    {
      free (pbuffer);
      return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
      if ((rssgu_status = rsWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
      {
        free (pbuffer);
        return rssgu_status;
      }

      if ((rssgu_status = rssgu_checkStatus (instrSession)) < VI_SUCCESS)
      {
        free (pbuffer);
        return rssgu_status;
      }

      if (count > (ViUInt32) binaryBlockLength)
      {
        rssgu_status = RSSGU_WARNING_SMALL_BUFFER;
        count = (ViUInt32) binaryBlockLength;
        memcpy (binaryBlockData, p2buffer, (size_t) count);
      }
      else
      {
        rssgu_status = VI_SUCCESS;
        memcpy (binaryBlockData, p2buffer, (size_t) count);
      }

      free (pbuffer);

      return rssgu_status;
  }


  /// HIFN This function reads out the subdirectories and files of the specified directory. If no directory is specified, the default directory selected with the MMEM:CDIR command is read out. The directories are output in a list (the list entries are separated by commas).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryPath/Directory path parameter.
  /// HIPAR arraySize/Array size of the parameter labeled 'Catalog List'.
  /// HIPAR catalogList/Reads back catalog list. Size of the list is limited by the 'Array Size' parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryCatalog (ViSession instrSession,
    ViString directoryPath,
    ViInt32 arraySize, ViChar catalogList[])
  {
    ViStatus    rssgu_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
      sprintf (buffer, "MMEM:CAT?");
    else
      sprintf (buffer, "MMEM:CAT? '%s'", directoryPath);

    rssgu_status = rssgu_getCatalog (instrSession,
      buffer,
      (ViUInt32) arraySize,
      catalogList,
      VI_ERROR_PARAMETER4);

    return rssgu_status;
  }
  
  /// HIFN This function reads out the number of entries in specified directory. If no directory is specified, the default directory selected with the MMEM:CDIR command is read out. The directories are output in a list (the list entries are separated by commas).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryPath/Directory path parameter.
  /// HIPAR length/Reads back number of entries int the catalog.
  ViStatus _VI_FUNC rssgu_MMEMoryCatalogLength (ViSession instrSession,
    ViString directoryPath,
    ViInt32* length)
  {
    ViStatus    rssgs_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
      sprintf (buffer, "MMEM:CAT:LENG?");
    else
      sprintf (buffer, "MMEM:CAT:LENG? '%s'", directoryPath);

    rssgs_status = rssgu_getViInt32(instrSession,
      buffer,
      length,
      VI_ERROR_PARAMETER3);

    return rssgs_status;
  }

  /// HIFN This function reads out the subdirectories of the specified directory. If no directory is specified, the default directory selected with the MMEM:CDIR command is read out. The directories are output in a list (the list entries are separated by commas).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryPath/Directory path parameter.
  /// HIPAR arraySize/Array size of the parameter labeled 'Catalog List'.
  /// HIPAR catalogList/Reads back catalog list. Size of the list is limited by the 'Array Size' parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryDirectoryCatalog (ViSession instrSession,
    ViString directoryPath,
    ViInt32 arraySize, ViChar _VI_FAR catalogList[])
  {
    ViStatus    rssgu_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
      sprintf (buffer, "MMEM:DCAT?");
    else
      sprintf (buffer, "MMEM:DCAT? '%s'", directoryPath);

    rssgu_status = rssgu_getCatalog (instrSession,
      buffer,
      (ViUInt32) arraySize,
      catalogList,
      VI_ERROR_PARAMETER4);

    return rssgu_status;
  }
  /// HIFN This function reads out the number of directories in specified directory. If no directory is specified, the default directory selected with the MMEM:CDIR command is read out. The directories are output in a list (the list entries are separated by commas).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryPath/Directory path parameter.
  /// HIPAR length/Reads back number of directories int the catalog.
  ViStatus _VI_FUNC rssgu_MMEMoryDirectoryCatalogLength (ViSession instrSession,
    ViString directoryPath,
    ViInt32* length)
  {
    ViStatus    rssgs_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
      sprintf (buffer, "MMEM:DCAT:LENG?");
    else
      sprintf (buffer, "MMEM:DCAT:LENG? '%s'", directoryPath);

    rssgs_status = rssgu_getViInt32(instrSession,
      buffer,
      length,
      VI_ERROR_PARAMETER3);

    return rssgs_status;
  }

  /// HIFN This function deletes the specified file.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR fileName/File name parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryDelete (ViSession instrSession, ViString fileName)
  {
    return rssgu_setViStringQuoted (instrSession,
      "MMEM:DEL",
      "'",
      fileName,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function creates a new subdirectory in the specified  directory. If no directory is specified, a subdirectory is created in the default directory. This function can also be used to create a directory tree.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryName/Directory name parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryMakeDirectory (ViSession instrSession,
    ViString directoryName)
  {
    return rssgu_setViStringQuoted (instrSession,
      "MMEM:MDIR",
      "'",
      directoryName,
      VI_ERROR_PARAMETER2);
  }


  /// HIFN This function renames an existing file ('File Source') if no path is specified for 'File Destination'. Otherwise the file is moved to the specified path and stored under the original file name or, if specified, a new file name. 
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR fileSource/Define file source.
  /// HIPAR fileDestination/Define file destination.
  ViStatus _VI_FUNC rssgu_MMEMoryMove (ViSession instrSession, ViString fileSource,
    ViString fileDestination)
  {
    ViStatus    rssgu_status;
    ViChar*     pbuffer;

    CHECKNULL (fileSource, 2);
    CHECKNULL (fileDestination, 3);

    pbuffer = (ViChar *)malloc (strlen (fileSource) + strlen (fileDestination) + 20);

    sprintf (pbuffer, "MMEM:MOVE '%s','%s'", fileSource, fileDestination);

    rssgu_status = rssgu_sendCmd (instrSession, pbuffer);

    free (pbuffer);

    RETURN;
  }

  /// HIFN This function deletes the specified subdirectory in the specified directory. If no directory is specified, the subdirectory with the specified name is deleted in the default directory.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR directoryName/Directory name parameter.
  ViStatus _VI_FUNC rssgu_MMEMoryRemoveDirectory (ViSession instrSession,
    ViString directoryName)
  {
    return rssgu_setViStringQuoted (instrSession,
      "MMEM:RDIR",
      "'",
      directoryName,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function loads the specified file stored under the specified name into internal memory
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR storage/number of the internal memory slot
  /// HIPAR fileName/file name from which to load the settings
  ViStatus _VI_FUNC rssgu_MMEMoryLoadState (ViSession instrSession,
    ViInt32 storage,  
    ViString fileName)
  {
    ViChar buffer[BUFFER_SIZE];
    sprintf (buffer, "MMEM:LOAD:STATE %d, \"%s\"\n", storage, fileName);
    return rsWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL);
  }

  /// HIFN This function stores the current instrument setting from internal memory in the specified file.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR storage/number of the internal memory slot
  /// HIPAR fileName/file name where to store the settings
  ViStatus _VI_FUNC rssgu_MMEMoryStoreState (ViSession instrSession,
    ViInt32 storage,  
    ViString fileName)
  {
    ViChar buffer[BUFFER_SIZE];
    sprintf (buffer, "MMEM:STORE:STATE %d, \"%s\"\n", storage, fileName);
    return rsWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL);
  }



  
  /// HIFN This function gets the result or current state of a selftest. Returns 0 on success.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR result/Returns result of the test. The response "0" indicates error-free testing, and the response "1" means that an error occurred during test.
  ViStatus _VI_FUNC rssgu_TESTSelftestResult (ViSession instrSession, ViInt32 *result)
  {
    return rssgu_getEnum (instrSession, 
      "TEST:ALL:RES?",
      result, 
      testResult, 
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function perform selftest. Returns 0 on success.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_TESTSelftestStart (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, "TEST:ALL:STARt");
  }
  
  
	//=====================================================================================
	/// HIFN This function configures frequency and level.
	/// HIRET Returns the status code of this operation.
	/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
	/// HIPAR frequency/RF frequency.
	/// HIPAR level/RF level.
	ViStatus _VI_FUNC rssgu_TestSpeed (ViSession instrSession, ViReal64  frequency, ViReal64  level )
	{
	  char buf[100] = {0};
	  sprintf (buf, "TEST:SPEED %f,%f\n", frequency, level);
	  return rssgu_sendCmd(instrSession,buf);
	}


  /// HIFN Enable/disable keyboard and LED test state..
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR state/Enable/disable keyboard and LED test state.
  ViStatus _VI_FUNC rssgu_SetTestKeyboardState (ViSession instrSession,
    ViBoolean state)
  {
    return rssgu_setViBoolean (instrSession, 
      "TEST:KEYB", 
      state,
      VI_ERROR_PARAMETER2);    
  }

  /// HIFN Returns keyboard and LED test state.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR state/Returns keyboard and LED test state.
  ViStatus _VI_FUNC rssgu_GetTestKeyboardState (ViSession instrSession,
    ViBoolean *state)
  {
    return rssgu_getViBoolean (instrSession, 
      "TEST:KEYB?", 
      state,
      VI_ERROR_PARAMETER2);    
  }

  /// HIFN This function sets the value of selected status register.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerGroup/Selects the status register group.
  /// HIPAR registerType/Selects the type of status register.
  /// HIPAR value/Sets the value to be set to selected register.
  ViStatus _VI_FUNC rssgu_SetRegister (ViSession instrSession, ViInt32 registerGroup,
    ViInt32 registerType, ViInt32 value)
  {
    ViChar  buffer[BUFFER_SIZE];

    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, registerGroup, 0, 1), 2);
    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, registerType, 2, 4), 3);

    sprintf (buffer, "STAT:%s:%s", regGroupArr[registerGroup], regTypeArr[registerType]);

    return rssgu_setViInt32 (instrSession,
      buffer,
      value,
      0, 0,
      VI_ERROR_PARAMETER4);
  }


  /// HIFN This function sets the value of selected status register.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR registerGroup/Selects the status register group.
  /// HIPAR registerType/Selects the type of status register.
  /// HIPAR value/Returns the value of selected register.
  ViStatus _VI_FUNC rssgu_GetRegister (ViSession instrSession, ViInt32 registerGroup,
    ViInt32 registerType, ViInt32* value)
  {
    ViChar  buffer[BUFFER_SIZE];

    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, registerGroup, 0, 1), 2);
    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, registerType, 0, 4), 3);

    sprintf (buffer, "STAT:%s:%s?", regGroupArr[registerGroup], regTypeArr[registerType]);

    return rssgu_getViInt32 (instrSession,
      buffer,
      value,
      VI_ERROR_PARAMETER4);
  }

  //=====================================================================================
  /// HIFN This function resets the status registers.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_STATusPRESet (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, 
      "STAT:PRES");
  }

  /// HIFN This function queries all entries in the error queue and then deletes them. Positive error numbers denote instrument-specific errors, and negative error numbers denote error messages defined by SCPI (see Chapter 9). If the error queue is empty, 0 ("No error") is returned.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/Array size of the parameter labeled 'Error Queue'.
  /// HIPAR errorQueue/Returns all entries in the error queue. Buffer length is limited by the 'Array Size' parameter.
  ViStatus _VI_FUNC rssgu_SYSTemErrorQueueAll (ViSession instrSession,
    ViInt32 arraySize, ViChar _VI_FAR errorQueue[])
  {
    return rssgu_getViString (instrSession,
      "SYST:ERR:ALL?",
      errorQueue,
      (ViUInt32) arraySize,
      VI_ERROR_PARAMETER3);
  }

  /// HIFN This function queries all entries in the error queue and then deletes them. Only the error numbers are returned and not the entire error text. Positive error numbers denote instrument-specific errors, and negative error numbers denote error messages defined by SCPI (see Chapter 9). If the error queue is empty, 0 ("No error") is returned.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/Array size of the parameter labeled 'Error Queue Code'.
  /// HIPAR errorQueueCode/Returns all entries in the error queue. Only the error numbers are returned and not the entire error text. Buffer length is limited by the 'Array Size' parameter.
  ViStatus _VI_FUNC rssgu_SYSTemErrorQueueCodeAll (ViSession instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR errorQueueCode[])
  {
    return rssgu_getViString (instrSession,
      "SYST:ERR:CODE:ALL?",
      errorQueueCode,
      (ViUInt32) arraySize,
      VI_ERROR_PARAMETER3);
  }

  /// HIFN This function queries the oldest entry in the error queue and then deletes it. Only the error number is returned and not the entire error text. Positive error numbers denote instrument-specific errors, and negative error numbers denote error messages defined by SCPI (see Chapter 9). If the error queue is empty, 0 is returned.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR errorQueueCodeNext/Returns the oldest entry in the error queue. Only the error number is returned and not the entire error text.
  ViStatus _VI_FUNC rssgu_SYSTemErrorQueueCodeNext (ViSession instrSession,
    ViInt32 *errorQueueCodeNext)
  {
    return rssgu_getViInt32 (instrSession,
      "SYST:ERR:CODE?",
      errorQueueCodeNext,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function queries the number of entries in the error queue. If the error queue is empty, '0' is returned.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR errorQueueCount/Returns the number of entries in the error queue.
  ViStatus _VI_FUNC rssgu_SYSTemErrorQueueCount (ViSession instrSession,
    ViInt32 *errorQueueCount)
  {
    return rssgu_getViInt32 (instrSession,
      "SYST:ERR:COUN?",
      errorQueueCount,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function returns a list of all errors existing at the time when the query is started. This list corresponds to the display on the info page under manual control.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/Array size of the parameter labeled 'Error Queue'.
  /// HIPAR errorQueue/Returns all entries in the error queue. Buffer length is limited by the 'Array Size' parameter.
  ViStatus _VI_FUNC rssgu_SYSTemStaticErrorQueue (ViSession instrSession,
    ViInt32 arraySize,
    ViChar errorQueue[])
  {
    return rssgu_getViString (instrSession,
      "SYST:SERR?",
      errorQueue,
      (ViUInt32) arraySize,
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function queries the USB resource.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR USBResource/This control display USB resource.
  ViStatus _VI_FUNC rssgu_SYSTemUSBResource (ViSession instrSession,
    ViUInt32 arraySize,
    ViChar USBResource[])
  {
    return rssgu_getViString (instrSession, 
      "SYST:COMM:USB:RES?", 
      USBResource, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function queries the Socket resource string.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR SocketResource/This control displays the Socket resource string.
  ViStatus _VI_FUNC rssgu_SYSTemSocketResource (ViSession instrSession,
    ViUInt32 arraySize,
    ViChar SocketResource[])
  {
    return rssgu_getViString (instrSession, 
      "SYST:COMM:SOCK:RES?", 
      SocketResource, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function queries the Socket resource string.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR PCIeResource/This control displays the PCIe resource string.
  ViStatus _VI_FUNC rssgu_SYSTemPCIeResource (ViSession instrSession,
    ViUInt32 arraySize,
    ViChar PCIeResource[])
  {
    return rssgu_getViString (instrSession, 
      "SYST:COMM:PCI:RES?", 
      PCIeResource, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function queries the Hislip resource string.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR HislipResource/This control displays the Hislip resource string.
  ViStatus _VI_FUNC rssgu_SYSTemHislipResource (ViSession instrSession,
    ViUInt32 arraySize,
    ViChar HislipResource[])
  {
    return rssgu_getViString (instrSession, 
      "SYST:COMM:HISL:RES?", 
      HislipResource, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }

  //=====================================================================================
  /// HIFN This function queries the serial resource string.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR arraySize/This control sets the array size.
  /// HIPAR SerialResource/This control displays the serial resource string.
  ViStatus _VI_FUNC rssgu_SYSTemSerialResource (ViSession instrSession,
    ViUInt32 arraySize,
    ViChar SerialResource[])
  {
    return rssgu_getViString (instrSession, 
      "SYST:COMM:SER:RES?", 
      SerialResource, 
      arraySize, 
      VI_ERROR_PARAMETER3);
  }


  /// HIFN This function restore the device state to the factory settings.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_SYSTemRestoreFactorySettings (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, "SYST:FPR");
  }
  
  

  /// HIFN  This function enables and selects the Eco Mode. With enabled eco mode 
  /// HIFN  EM1 the doubler stage in a 12 GHz instrument is permanently switched 
  /// HIFN  off to reduce power consumption and the maximum frequency is limited to 
  /// HIFN  6 GHz.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
  /// HIPAR instrSession/function to select the desired instrument driver session.
  /// HIPAR mode/This control selects the Eco Mode..
  ViStatus _VI_FUNC rssgu_SetSYSTemEcoMode (ViSession instrSession,
    ViInt32 mode)
  {
    return rssgu_setEnum (instrSession,
      "SYST:EMOD",
      mode,
      ecoModeArr,
      VI_ERROR_PARAMETER2);    
  }

  /// HIFN  This function returns and selects the Eco Mode. With enabled eco mode 
  /// HIFN  EM1 the doubler stage in a 12 GHz instrument is permanently switched 
  /// HIFN  off to reduce power consumption and the maximum frequency is limited to 
  /// HIFN  6 GHz.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
  /// HIPAR instrSession/function to select the desired instrument driver session.
  /// HIPAR mode/This control returns the Eco Mode.
  ViStatus _VI_FUNC rssgu_GetSYSTemEcoMode (ViSession instrSession,
    ViInt32 *mode)
  {
    return rssgu_getEnum (instrSession,
      "SYST:EMOD?",
      mode,
      ecoModeArr,
      VI_ERROR_PARAMETER2);    
  }

  //=====================================================================================
  /// HIFN This function activates and deactivates the specified protection level.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR protectionLevel/Specify protection level.
  /// HIPAR protection/Activates and deactivates the specified protection level.
  /// HIPAR password/The password for the specified protection level.
  ViStatus _VI_FUNC rssgu_SetSYSTemProtect (ViSession instrSession,
    ViInt32 protectionLevel,
    ViBoolean protection,
    ViString password)
  {
    ViChar  buffer [BUFFER_SIZE];

    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, protectionLevel, 1, 5), 2);
    CHECKPARAM (rssgu_invalidViBooleanRange (instrSession, protection), 3);
    CHECKNULL (password, 4);

    if (protection == VI_FALSE)
      sprintf (buffer, "SYST:PROT%ld:STAT OFF,%s", protectionLevel, password);
    else
      sprintf (buffer, "SYST:PROT%ld:STAT ON", protectionLevel);

    return rssgu_sendCmd (instrSession, buffer);
  }

  //=====================================================================================
  /// HIFN This function queries state of the specified protection level.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR protectionLevel/Specify protection level.
  /// HIPAR protection/Returns state of the specified protection level.
  ViStatus _VI_FUNC rssgu_GetSYSTemProtect (ViSession instrSession,
    ViInt32 protectionLevel,
    ViBoolean *protection)
  {
    ViChar  buffer [BUFFER_SIZE];

    CHECKPARAM (rssgu_invalidViInt32Range (instrSession, protectionLevel, 1, 5), 2);
    sprintf (buffer, "SYST:PROT%ld:STAT?", protectionLevel);

    return rssgu_getViBoolean (instrSession, 
      buffer,
      protection,
      VI_ERROR_PARAMETER3);
  }

  /// HIFN This function queries the SCPI version with which the instrument complies.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR SCPIVersion/Returns the SCPI version with which the instrument complies.
  ViStatus _VI_FUNC rssgu_GetSYSTemVersion (ViSession instrSession,
    ViChar _VI_FAR SCPIVersion[])
  {
    ViStatus rssgu_status = VI_SUCCESS;

    rssgu_status = rssgu_getViString (instrSession,
      "SYST:VERS?",
      SCPIVersion,
      32,
      VI_ERROR_PARAMETER2);

    return (rssgu_status < VI_SUCCESS) ? rssgu_status : VI_SUCCESS;
  }

  //=====================================================================================
  /// HIFN This function returns if the startup of the instrument is completed.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR startupComplete/This control displays the startup of the instrument is completed.
  ViStatus _VI_FUNC rssgu_GetSYSTemStartupStatus (ViSession instrSession,
    ViInt32 *startupComplete)
  {
    return rssgu_getViInt32 (instrSession, 
      "SYST:STAR:COMP?", 
      startupComplete, 
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function sets the Terminator mode for the IEC bus.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR mode/This control sets the Terminator mode for the IEC bus.
  ViStatus _VI_FUNC rssgu_SetSYSTemTerminatorMode (ViSession instrSession,
    ViInt32 mode)
  {
    return rssgu_setEnum (instrSession,
      "SYST:COMM:GPIB:LTER",
      mode,
      terminatorModeArr,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function queries the Terminator mode for the IEC bus.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR mode/Displays the Terminator mode for the IEC bus.
  ViStatus _VI_FUNC rssgu_GetSYSTemTerminatorMode (ViSession instrSession,
    ViInt32 *mode)
  {
    return rssgu_getEnum (instrSession,
      "SYST:COMM:GPIB:LTER?",
      mode,
      terminatorModeArr,
      VI_ERROR_PARAMETER2);
  }





  

  /// HIFN This function restarts the device.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_Restart (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, "REST");
  }

  /// HIFN This function puts the device in the standby state.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_Standby (ViSession instrSession)
  {
    return rssgu_sendCmd (instrSession, "STAN");
  }

  //=====================================================================================
  /// HIFN This function sets the default unit for angles.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR angleUnits/Sets the default unit for angles.
  ViStatus _VI_FUNC rssgu_SetAngleUnits (ViSession instrSession,
    ViInt32 angleUnits)
  {
    return rssgu_setEnum (instrSession, 
      "UNIT:ANGL", 
      angleUnits,
      AngleUnitsArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function returns the unit for angles.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR angleUnits/Gets the unit for angles.
  ViStatus _VI_FUNC rssgu_GetAngleUnits (ViSession instrSession,
    ViInt32 *angleUnits)
  {
    return rssgu_getEnum (instrSession, 
      "UNIT:ANGL?", 
      angleUnits,
      AngleUnitsArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function sets the default unit for power.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR powerUnits/Sets the default unit for power.
  ViStatus _VI_FUNC rssgu_SetPowerUnits (ViSession instrSession,
    ViInt32 powerUnits)
  {
    return rssgu_setEnum (instrSession, 
      "UNIT:POW", 
      powerUnits,
      PowerUnitsArr, 
      VI_ERROR_PARAMETER2);
  }

  //=====================================================================================
  /// HIFN This function returns the unit for power.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR powerUnits/Gets the unit for power.
  ViStatus _VI_FUNC rssgu_GetPowerUnits (ViSession instrSession,
    ViInt32 *powerUnits)
  {
    return rssgu_getEnum (instrSession, 
      "UNIT:POW?", 
      powerUnits,
      PowerUnitsArr, 
      VI_ERROR_PARAMETER2);
  }


  /// HIFN This function sets the data format for IEC bus.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR format/Sets the data format for IEC bus.
  ViStatus _VI_FUNC rssgu_SetFormatData (ViSession instrSession, ViInt32 format)
  {
    return rssgu_setEnum (instrSession,
      "FORM:DATA",
      format,
      formatDataArr,
      VI_ERROR_PARAMETER2);
  }

  /// HIFN This function queries the data format for IEC bus.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR format/Displays the data format for IEC bus.
  ViStatus _VI_FUNC rssgu_GetFormatData (ViSession instrSession, ViInt32 *format)
  {
    return rssgu_getEnum (instrSession,
      "FORM:DATA?",
      format,
      formatDataArr,
      VI_ERROR_PARAMETER2);
  }


  /// HIFN This function switches state checking of the instrument (reading of the Standard Event Register and checking it for error) status subsystem. Driver functions are using state checking which is by default enabled.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR state/This control switches instrument state checking On or Off.
  ViStatus _VI_FUNC rssgu_errorCheckState (ViSession instrSession,
    ViBoolean state)
  {
    ViStatus    rssgu_status = VI_SUCCESS;
    rssgu_instrRange instrPtr = NULL;
    unsigned int     ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    CHECKNULL (instrPtr, 1);

    instrPtr -> errorChecking = state;

    return rssgu_status;
  }

  /// HIFN This function switches option checking of the instrument.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR state/This control switches instrument and option checking On or Off.
  ViStatus _VI_FUNC rssgu_optionCheckState (ViSession instrSession,
    ViBoolean state)
  {
    ViStatus    rssgu_status = VI_SUCCESS;
    rssgu_instrRange instrPtr = NULL;
    unsigned int        ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    if (instrPtr != NULL)
      instrPtr -> optionChecking = state;

    return rssgu_status;

  }

  /// HIFN This function switches range checking of the instrument.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR rangeChecking/This control switches instrument and range checking On or Off.
  ViStatus _VI_FUNC rssgu_setCheckRange(
    ViSession   instrSession,
    ViBoolean   rangeChecking
    )
  {
    ViStatus    rssgu_status = VI_SUCCESS;
    rssgu_instrRange instrPtr;
    unsigned int     ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    instrPtr -> rangeChecking = rangeChecking;

    return rssgu_status;
  }

  /// HIFN This function writes commands and queries to the instrument to modify parameters and query device settings.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR writeBuffer/The user can use this control to send common commands and queries to the instrument. This control can also be used to write any valid command to the instrument.
  ViStatus _VI_FUNC rssgu_writeInstrData (ViSession instrSession, ViString writeBuffer)
  {
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR (rsPrintf (instrSession, "%s\n", writeBuffer));

    return rssgu_status;
  }

  /// HIFN This function reads data from the instrument's output buffer and returns it to the specified variable in memory.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR numberBytesToRead/The number of bytes specified by this control should be greater than or equal to the number of bytes which are to be read from the instrument. If the actual number of bytes to be read is greater than the number this control specifies then multiple reads will be required to empty the instrument's output buffer.
  /// HIPAR readBuffer/The incoming data from the instrument is placed into this variable.
  /// HIPAR numBytesRead/This variable contains the actual number of bytes read from the instrument. This is the value which is returned by the read function.
  ViStatus _VI_FUNC rssgu_readInstrData (ViSession instrSession,
    ViInt32 numberBytesToRead,
    ViChar _VI_FAR readBuffer[],
    ViPInt32 numBytesRead)
  {
    ViStatus    rssgu_status = VI_SUCCESS;
    ViUInt32    tmpCnt = 0;

    if (numBytesRead != NULL)
      *numBytesRead = 0L;

    CHECKERR (rsRead (instrSession, (ViBuf)readBuffer, (ViUInt32) numberBytesToRead, &tmpCnt));

    if (numBytesRead)
      *numBytesRead = (ViInt32)tmpCnt;

    return rssgu_status;
  }

  
  /// HIFN RESET resets the instrument to a defined default state. The function has the same effect as pressing the PRESET key.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_reset (ViSession instrSession)
  {
    ViStatus rssgu_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (rsPrintf (instrSession, "*RST;*CLS\n"));
    CHECKERR (rssgu_defaultInstrSetup (instrSession));

    return rssgu_status;
  }

  //=====================================================================================
  /// HIFN clears the device's input and output buffers
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  ViStatus _VI_FUNC rssgu_Clear (ViSession instrSession)
  {
    return rsClear(instrSession);
  }

  /// HIFN This function runs the instrument's self test routine and returns the test result(s).
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR testResult/This control contains the value returned from the instrument self test. Zero means success. For any other code, see the device's operator's manual.
  /// HIPAR testMessage/This control contains the string returned from the self test. See the device's operation manual for an explanation of the string's contents.
  ViStatus _VI_FUNC rssgu_self_test (ViSession instrSession,
    ViPInt16 testResult,
    ViChar _VI_FAR testMessage[])
  {
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR( rsQueryf (instrSession, "*TST?\n", "%hd", testResult));

    sprintf (testMessage, "%s", (*testResult == 0) ? "Self-Test Passed" : "Self-Test Failed");

    return rssgu_status;
  }

  /// HIFN This function reads an error code from the instrument's error queue.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR errorCode/This control returns the error code read from the instrument's error queue.
  /// HIPAR errorMessage/This control returns the error message string read from the instrument's error message queue.
  ViStatus _VI_FUNC rssgu_error_query (ViSession instrSession,
    ViPInt32 errorCode,
    ViChar _VI_FAR errorMessage[])
  {
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR (rsPrintf (instrSession, ":SYST:ERR?\n"));
    CHECKERR (rsScanf (instrSession, "%ld,\"%[^\"]", errorCode, errorMessage));

    return rssgu_status;
  }

  /// HIFN This function takes the Status Code returned by the instrument driver functions, interprets it and returns it as a user readable string.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR statusCode/This control accepts the Status Code returned from the instrument driver functions.
  /// HIPAR errMessage/This control returns the interpreted Status Code as a user readable message string.
  ViStatus _VI_FUNC rssgu_error_message (ViSession instrSession,
    ViStatus statusCode,
    ViChar _VI_FAR errMessage[])
  {
    ViStatus rssgu_status = VI_WARN_UNKNOWN_STATUS;
    ViInt16 i;

    static rssgu_tStringValPair statusDescArray[] = {
      {VI_SUCCESS,                            "No Error"},
      {VI_WARN_NSUP_ID_QUERY,                 "WARNING: ID Query not supported"},
      {VI_WARN_NSUP_RESET,                    "WARNING: Reset not supported"},
      {VI_WARN_NSUP_SELF_TEST,                "WARNING: Self-test not supported"},
      {VI_WARN_NSUP_ERROR_QUERY,              "WARNING: Error Query not supported"},
      {VI_WARN_NSUP_REV_QUERY,                "WARNING: Revision Query not supported"},
      {VI_ERROR_PARAMETER1,                   "ERROR: Parameter 1 out of range"},
      {VI_ERROR_PARAMETER2,                   "ERROR: Parameter 2 out of range"},
      {VI_ERROR_PARAMETER3,                   "ERROR: Parameter 3 out of range"},
      {VI_ERROR_PARAMETER4,                   "ERROR: Parameter 4 out of range"},
      {VI_ERROR_PARAMETER5,                   "ERROR: Parameter 5 out of range"},
      {VI_ERROR_PARAMETER6,                   "ERROR: Parameter 6 out of range"},
      {VI_ERROR_PARAMETER7,                   "ERROR: Parameter 7 out of range"},
      {VI_ERROR_PARAMETER8,                   "ERROR: Parameter 8 out of range"},
      {VI_ERROR_FAIL_ID_QUERY,                "ERROR: Identification query failed"},
      {VI_ERROR_INV_RESPONSE,                 "ERROR: Interpreting instrument response"},
      {VI_ERROR_INSTR_FILE_OPEN,              "ERROR: Opening the specified file"},
      {VI_ERROR_INSTR_FILE_WRITE,             "ERROR: Writing to the specified file"},
      {VI_ERROR_INSTR_INTERPRETING_RESPONSE,  "ERROR: Interpreting the instrument's response"},
      {VI_ERROR_INSTR_PARAMETER8,             "ERROR: Parameter 8 out of range"},
      {VI_ERROR_INSTR_PARAMETER9,             "ERROR: Parameter 9 out of range"},
      {VI_ERROR_INSTR_PARAMETER10,            "ERROR: Parameter 10 out of range"},
      {VI_ERROR_INSTR_PARAMETER11,            "ERROR: Parameter 11 out of range"},
      {VI_ERROR_INSTR_PARAMETER12,            "ERROR: Parameter 12 out of range"},
      {VI_ERROR_INSTR_PARAMETER13,            "ERROR: Parameter 13 out of range"},
      {VI_ERROR_INSTR_PARAMETER14,            "ERROR: Parameter 14 out of range"},
      {VI_ERROR_INSTR_PARAMETER15,            "ERROR: Parameter 15 out of range"},

      {VI_SUCCESS_EVENT_EN                    ,"VI_SUCCESS_EVENT_EN"},
      {VI_SUCCESS_EVENT_DIS                   ,"VI_SUCCESS_EVENT_DIS"},
      {VI_SUCCESS_QUEUE_EMPTY                 ,"VI_SUCCESS_QUEUE_EMPTY"},
      {VI_SUCCESS_TERM_CHAR                   ,"VI_SUCCESS_TERM_CHAR"},                  
      {VI_SUCCESS_MAX_CNT                     ,"VI_SUCCESS_MAX_CNT"},                    
      {VI_SUCCESS_DEV_NPRESENT                ,"VI_SUCCESS_DEV_NPRESENT"},
      {VI_SUCCESS_TRIG_MAPPED                 ,"VI_SUCCESS_TRIG_MAPPED"},
      {VI_SUCCESS_QUEUE_NEMPTY                ,"VI_SUCCESS_QUEUE_NEMPTY"},
      {VI_SUCCESS_NCHAIN                      ,"VI_SUCCESS_NCHAIN"},
      {VI_SUCCESS_NESTED_SHARED               ,"VI_SUCCESS_NESTED_SHARED"},
      {VI_SUCCESS_NESTED_EXCLUSIVE            ,"VI_SUCCESS_NESTED_EXCLUSIVE"},
      {VI_SUCCESS_SYNC                        ,"VI_SUCCESS_SYNC"},
      {VI_WARN_CONFIG_NLOADED                 ,"VI_WARN_CONFIG_NLOADED"},
      {VI_WARN_NULL_OBJECT                    ,"VI_WARN_NULL_OBJECT"},
      {VI_WARN_NSUP_ATTR_STATE                ,"VI_WARN_NSUP_ATTR_STATE"},
      {VI_WARN_UNKNOWN_STATUS                 ,"VI_WARN_UNKNOWN_STATUS"},
      {VI_WARN_NSUP_BUF                       ,"VI_WARN_NSUP_BUF"},
      {VI_ERROR_SYSTEM_ERROR                  ,"VI_ERROR_SYSTEM_ERROR"},
      {VI_ERROR_INV_OBJECT                    ,"VI_ERROR_INV_OBJECT"},
      {VI_ERROR_RSRC_LOCKED                   ,"VI_ERROR_RSRC_LOCKED"},
      {VI_ERROR_INV_EXPR                      ,"VI_ERROR_INV_EXPR"},
      {VI_ERROR_RSRC_NFOUND                   ,"VI_ERROR_RSRC_NFOUND"},
      {VI_ERROR_INV_RSRC_NAME                 ,"VI_ERROR_INV_RSRC_NAME"},
      {VI_ERROR_INV_ACC_MODE                  ,"VI_ERROR_INV_ACC_MODE"},
      {VI_ERROR_TMO                           ,"VI_ERROR_TMO"},
      {VI_ERROR_CLOSING_FAILED                ,"VI_ERROR_CLOSING_FAILED"},
      {VI_ERROR_INV_DEGREE                    ,"VI_ERROR_INV_DEGREE"},
      {VI_ERROR_INV_JOB_ID                    ,"VI_ERROR_INV_JOB_ID"},
      {VI_ERROR_NSUP_ATTR                     ,"VI_ERROR_NSUP_ATTR"},
      {VI_ERROR_NSUP_ATTR_STATE               ,"VI_ERROR_NSUP_ATTR_STATE"},
      {VI_ERROR_ATTR_READONLY                 ,"VI_ERROR_ATTR_READONLY"},
      {VI_ERROR_INV_LOCK_TYPE                 ,"VI_ERROR_INV_LOCK_TYPE"},
      {VI_ERROR_INV_ACCESS_KEY                ,"VI_ERROR_INV_ACCESS_KEY"},
      {VI_ERROR_INV_EVENT                     ,"VI_ERROR_INV_EVENT"},
      {VI_ERROR_INV_MECH                      ,"VI_ERROR_INV_MECH"},
      {VI_ERROR_HNDLR_NINSTALLED              ,"VI_ERROR_HNDLR_NINSTALLED"},
      {VI_ERROR_INV_HNDLR_REF                 ,"VI_ERROR_INV_HNDLR_REF"},
      {VI_ERROR_INV_CONTEXT                   ,"VI_ERROR_INV_CONTEXT"},
      {VI_ERROR_QUEUE_OVERFLOW                ,"VI_ERROR_QUEUE_OVERFLOW"},
      {VI_ERROR_NENABLED                      ,"VI_ERROR_NENABLED"},
      {VI_ERROR_ABORT                         ,"VI_ERROR_ABORT"},
      {VI_ERROR_RAW_WR_PROT_VIOL              ,"VI_ERROR_RAW_WR_PROT_VIOL"},
      {VI_ERROR_RAW_RD_PROT_VIOL              ,"VI_ERROR_RAW_RD_PROT_VIOL"},
      {VI_ERROR_OUTP_PROT_VIOL                ,"VI_ERROR_OUTP_PROT_VIOL"},
      {VI_ERROR_INP_PROT_VIOL                 ,"VI_ERROR_INP_PROT_VIOL"},
      {VI_ERROR_BERR                          ,"VI_ERROR_BERR"},
      {VI_ERROR_IN_PROGRESS                   ,"VI_ERROR_IN_PROGRESS"},
      {VI_ERROR_INV_SETUP                     ,"VI_ERROR_INV_SETUP"},
      {VI_ERROR_QUEUE_ERROR                   ,"VI_ERROR_QUEUE_ERROR"},
      {VI_ERROR_ALLOC                         ,"VI_ERROR_ALLOC"},
      {VI_ERROR_INV_MASK                      ,"VI_ERROR_INV_MASK"},
      {VI_ERROR_IO                            ,"VI_ERROR_IO"},
      {VI_ERROR_INV_FMT                       ,"VI_ERROR_INV_FMT"},
      {VI_ERROR_NSUP_FMT                      ,"VI_ERROR_NSUP_FMT"},
      {VI_ERROR_LINE_IN_USE                   ,"VI_ERROR_LINE_IN_USE"},
      {VI_ERROR_NSUP_MODE                     ,"VI_ERROR_NSUP_MODE"},
      {VI_ERROR_SRQ_NOCCURRED                 ,"VI_ERROR_SRQ_NOCCURRED"},
      {VI_ERROR_INV_SPACE                     ,"VI_ERROR_INV_SPACE"},
      {VI_ERROR_INV_OFFSET                    ,"VI_ERROR_INV_OFFSET"},
      {VI_ERROR_INV_WIDTH                     ,"VI_ERROR_INV_WIDTH"},
      {VI_ERROR_NSUP_OFFSET                   ,"VI_ERROR_NSUP_OFFSET"},
      {VI_ERROR_NSUP_VAR_WIDTH                ,"VI_ERROR_NSUP_VAR_WIDTH"},
      {VI_ERROR_WINDOW_NMAPPED                ,"VI_ERROR_WINDOW_NMAPPED"},
      {VI_ERROR_RESP_PENDING                  ,"VI_ERROR_RESP_PENDING"},
      {VI_ERROR_NLISTENERS                    ,"VI_ERROR_NLISTENERS"},
      {VI_ERROR_NCIC                          ,"VI_ERROR_NCIC"},
      {VI_ERROR_NSYS_CNTLR                    ,"VI_ERROR_NSYS_CNTLR"},
      {VI_ERROR_NSUP_OPER                     ,"VI_ERROR_NSUP_OPER"},
      {VI_ERROR_INTR_PENDING                  ,"VI_ERROR_INTR_PENDING"},
      {VI_ERROR_ASRL_PARITY                   ,"VI_ERROR_ASRL_PARITY"},
      {VI_ERROR_ASRL_FRAMING                  ,"VI_ERROR_ASRL_FRAMING"},
      {VI_ERROR_ASRL_OVERRUN                  ,"VI_ERROR_ASRL_OVERRUN"},
      {VI_ERROR_TRIG_NMAPPED                  ,"VI_ERROR_TRIG_NMAPPED"},
      {VI_ERROR_NSUP_ALIGN_OFFSET             ,"VI_ERROR_NSUP_ALIGN_OFFSET"},
      {VI_ERROR_USER_BUF                      ,"VI_ERROR_USER_BUF"},
      {VI_ERROR_RSRC_BUSY                     ,"VI_ERROR_RSRC_BUSY"},
      {VI_ERROR_NSUP_WIDTH                    ,"VI_ERROR_NSUP_WIDTH"},
      {VI_ERROR_INV_PARAMETER                 ,"VI_ERROR_INV_PARAMETER"},
      {VI_ERROR_INV_PROT                      ,"VI_ERROR_INV_PROT"},
      {VI_ERROR_INV_SIZE                      ,"VI_ERROR_INV_SIZE"},
      {VI_ERROR_WINDOW_MAPPED                 ,"VI_ERROR_WINDOW_MAPPED"},
      {VI_ERROR_NIMPL_OPER                    ,"VI_ERROR_NIMPL_OPER"},
      {VI_ERROR_INV_LENGTH                    ,"VI_ERROR_INV_LENGTH"},
      {VI_ERROR_INV_MODE                      ,"VI_ERROR_INV_MODE"},
      {VI_ERROR_SESN_NLOCKED                  ,"VI_ERROR_SESN_NLOCKED"},
      {VI_ERROR_MEM_NSHARED                   ,"VI_ERROR_MEM_NSHARED"},
      {VI_ERROR_LIBRARY_NFOUND                ,"VI_ERROR_LIBRARY_NFOUND"},
      {VI_ERROR_NSUP_INTR                     ,"VI_ERROR_NSUP_INTR"},
      {VI_ERROR_INV_LINE                      ,"VI_ERROR_INV_LINE"},
      {VI_ERROR_FILE_ACCESS                   ,"VI_ERROR_FILE_ACCESS"},
      {VI_ERROR_FILE_IO                       ,"VI_ERROR_FILE_IO"},
      {VI_ERROR_NSUP_LINE                     ,"VI_ERROR_NSUP_LINE"},
      {VI_ERROR_NSUP_MECH                     ,"VI_ERROR_NSUP_MECH"},
      {VI_ERROR_INTF_NUM_NCONFIG              ,"VI_ERROR_INTF_NUM_NCONFIG"},
      {VI_ERROR_CONN_LOST                     ,"VI_ERROR_CONN_LOST"},
      {VI_ERROR_NPERMISSION                   ,"VI_ERROR_NPERMISSION"},


      /* --- Instrument specific error codes --- */

      {RSSGU_ERROR_INSTRUMENT_ERROR,          "ERROR: Instrument status error"},
      {RSSGU_ERROR_INVALID_CONFIGURATION,     "ERROR: Instrument configuration error"},
      {RSSGU_ERROR_INSTRUMENT_OPTION,         "ERROR: Option not installed"},
      {RSSGU_ERROR_INSTRUMENT_NSUP_MODEL,     "ERROR: Required Instrument model is not available."},
      {RSSGU_ERROR_SETTINGS_CONFLICT,         "ERROR: Instrument settings conflict."},
      {RSSGU_ERROR_CALIBRATION_FAILED,        "ERROR: Calibration failed."},
      {RSSGU_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE, "ERROR: Data not available."},

      /* --- Instrument specific warnings --- */

      {RSSGU_WARNING_SMALL_BUFFER,            "WARNING: Buffer too small"},

      {VI_NULL, VI_NULL}
    };
    sprintf (errMessage, "Unknown Error 0x%X", (unsigned int) statusCode);
    rssgu_status = rsStatusDesc (instrSession, statusCode, errMessage);
    if (rssgu_status == VI_SUCCESS)
      return (VI_SUCCESS);

    for (i = 0; statusDescArray[i].stringName; i++)
    {
      if (statusDescArray[i].stringVal == statusCode)
      {
        sprintf (errMessage, "%s", statusDescArray[i].stringName);
        return (VI_SUCCESS);
      }
    }
    printf ("Unknown Error 0x%X", (unsigned int) statusCode);

    return VI_WARN_UNKNOWN_STATUS;
  }

  /// HIFN This function returns the revision numbers of the instrument driver and instrument firmware, and tells the user with which instrument firmware this revision of the driver is compatible.
  /// HIRET Returns the status code of this operation.
  /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
  /// HIPAR driverRev/This control returns the Instrument Driver Software Revision.
  /// HIPAR instrRev/This control returns the Instrument Firmware Revision.
  ViStatus _VI_FUNC rssgu_revision_query (ViSession instrSession,
    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
  {
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR (rsPrintf (instrSession, "*IDN?\n"));
    //CHECKERR (rsScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));
    CHECKERR (rsScanf (instrSession, "%*[^-]-%[^ (\r\n]", instrRev));

    sprintf (driverRev, "%s", RSSGU_REVISION);

    return rssgu_status;
  }

  /// HIFN Find devices on the PCIe bus and return their serial numbers
  /// HIRET Error code
  /// HIPAR vendorId/PCI id of the device's manufacturer
  /// HIPAR deviceId/PCI id of the device
  /// HIPAR serialNumbers/array with returned serial numbers. Allocated by caller
  /// HIPAR serialNumberCount/allocated size of array. On return: number of found devices
  ViStatus _VI_FUNC rssgu_enumerate_pcie_devices (ViInt32 vendorId, ViInt32 deviceId, ViInt32 serialNumbers[], ViPInt32 serialNumberCount)
  {
    ViStatus status = viEnumerate_pcie_devices (vendorId, deviceId, serialNumbers, serialNumberCount);
    return status;
  }

/// HIFN Find fast socket devices and return their types, serial numbers
/// HIFN and IP addresses. Call function with infoCount=0/info = NULL,
/// HIFN allocate buffer and call again with valid buffers.  
/// HIRET Returns the status code of this operation.
/// HIPAR info/Array of pointers to string buffers. Each buffer must be at least 64 bytes long.
/// HIPAR info/Each string returned by the function is a comma-separated list of instrument type, 
/// HIPAR info/serial number and IP address e.g. SGT100A,100027,169.254.45.172. 
/// HIPAR infoCount/Number of buffers in array info. On return: the number of found devices.
ViStatus _VI_FUNC rssgu_enumerate_socket_devices (ViPString *info, ViPInt32 infoCount)                                               
  {
    ViStatus status = viEnumerate_socket_devices (info, infoCount);
    return status;
  }

  /*===========================================================================*/
  /* Function: Delay                                                           */
  /* Purpose : With a call to rssgu_delay, the current program is suspended    */
  /*           from execution for the number of seconds specified by the       */
  /*           argument seconds. rssgu_delay is accurate to a millisecond.     */
  /*           This function is platform independent.                          */
  /*===========================================================================*/
  ViStatus _VI_FUNC rssgu_delay (/*@unused@*/ViSession instrSession, ViReal64 seconds)
  {
    clock_t StartTime = clock();

    while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (seconds)){
#ifdef _WIN32
      Sleep(0);
#endif
    }
    return VI_SUCCESS;
  }

  /*===========================================================================*/
  /* Function: Close                                                           */
  /* Purpose:  This function closes the instrument.                            */
  /*===========================================================================*/
  ViStatus _VI_FUNC rssgu_close (ViSession instrSession)
  {
    rssgu_instrRange instrPtr = NULL;
    ViSession        rmSession = 0;
    ViStatus         rssgu_status = VI_SUCCESS;
    unsigned int     ptrHandle = 0xffffffff;

    CHECKERR( rsGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);
    rssgu_RemovePointer(ptrHandle);

    if (instrPtr != NULL)
      free (instrPtr);

    rssgu_status = rsClose (instrSession);

    return rssgu_status;
  }

  /*===========================================================================*/
  /*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
  /*===========================================================================*/

  /*===========================================================================*/
  /* Function: Default Instrument Setup                                        */
  /* Purpose:  This function sends a default setup to the instrument.  This    */
  /*           function is called by the rssgu_reset operation and by the     */
  /*           rssgu_init function if the reset option has not been           */
  /*           selected.  This function is useful for configuring any          */
  /*           instrument settings that are required by the rest of the        */
  /*           instrument driver functions such as turning headers ON or OFF   */
  /*           or using the long or short form for commands, queries, and data.*/
  /*===========================================================================*/
  ViStatus rssgu_defaultInstrSetup (ViSession instrSession)
  {
    ViStatus            rssgu_status = VI_SUCCESS;
    rssgu_instrRange    instrPtr = NULL;
    unsigned int        ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    CHECKNULL (instrPtr, 1);

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    instrPtr -> errorChecking = VI_TRUE;
    instrPtr -> optionChecking = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, RSSGU_REVISION);

    return rssgu_status;
  }

  /*===========================================================================*/
  /* Function: Initialize Clean Up                                             */
  /* Purpose:  This function is used only by the rssgu_init function.  When   */
  /*           an error is detected this function is called to close the       */
  /*           open resource manager and instrument object sessions and to     */
  /*           set the instrSession that is returned from rssgu_init to       */
  /*           VI_NULL.                                                        */
  /*===========================================================================*/
  ViStatus rssgu_initCleanUp (ViSession openRMSession,
    ViPSession openInstrSession,
    ViStatus currentStatus)
  {
    unsigned int        ptrHandle = 0xffffffff;

    if (rsGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &ptrHandle) >= 0)
    {
      rssgu_instrRange instrPtr = NULL;
      instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);
      if (instrPtr != NULL)
        free (instrPtr);

    }

    (void) rsClose (*openInstrSession);
    (void) rsClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
  }

