/*===========================================================================*/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*===========================================================================*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "rssgu.h"
#include "rssgu_utility.h"

#include "visaabstraction.h"

#define IEEE_ERROR_BITS         0x3C    /* AND mask for ESR checking */
#define BUFFER_SIZE             2048UL    /* File I/O buffer size */

#define MAXPTRHANDLES 100
static void* g_PointerList[MAXPTRHANDLES] = {0};


static unsigned int rssgu_FindNextFreePointerEntry()
{
  int i = 0;
  for (i = 0; i < MAXPTRHANDLES; i++)
  {
    if (g_PointerList[i] == 0)
      return i;
  }
  return MAXPTRHANDLES;
}

unsigned int rssgu_AddPointer(void* pPtr)
{
  unsigned int index = rssgu_FindNextFreePointerEntry();
  if (index < MAXPTRHANDLES)
  {
    g_PointerList[index] = pPtr;
  }
  else
  {
    printf ("Pointer list full\n");
  }
  return index;
}

void rssgu_RemovePointer(unsigned int handle)
{
  g_PointerList[handle] = 0;
}

void* rssgu_GetPointer(unsigned int handle)
{
  if (handle < MAXPTRHANDLES)
    return g_PointerList[handle];

  return 0;
}


/*===========================================================================*/
/* Function:  f2a*/
/* Purpose:  Convert a floating point value into an ASCII string             */
/*             value[in]: value to be converted                              */
/*             szOut[out]: pointer to character buffer allocated by caller   */ 
/*                         holds result                                      */
/*             value[in]: value to be converted                              */
/*===========================================================================*/

static void f2a(double value, char* szOut, int nCnt)
{
  char txt[100];
  char* p = 0;
  char* pStart = 0;
  char *pDpoint = 0;

	double  dExpo = 0;
	if (value != 0)
		dExpo = log10(fabs(value));

  if( fabs (dExpo) < 32)
  {
    const int nMaxSignificantDigits = 15;
    char szFormat[100];

    int nLsDigit = (int)ceil (dExpo - nMaxSignificantDigits);
    if (nLsDigit < 0)
    {
      sprintf (szFormat, "%%1.%df", -nLsDigit);    
      sprintf(txt,szFormat, value);
    }
    else
    {
      char Zeros[] = {"00000000000000000000000000000000000000"};
      const double v = pow (10.0,nMaxSignificantDigits);

      double ll = floor(value / v + 0.5);

      strcpy (szFormat, "%f");    
      Zeros[nMaxSignificantDigits] = 0;
      sprintf(txt,szFormat, ll);
      p = strchr(txt,'.');
      if (p)
        *p= 0;
      strcat (txt,Zeros);

    }
  }
  else
  {
    sprintf(txt,"%g",value);
  }

  pStart = txt;
  while (*pStart == ' ') // skip leading blanks
    pStart++;

  pDpoint = strchr (pStart,'.');
  if (pDpoint)
  {
    char* pC = pStart+strlen(pStart)-1;
    while (*pC == '0') // truncate trailing zeros in the fractional part
    {
      *pC = 0;
      pC--;
    }  

    // kill trailing .
    pC = pDpoint+1; 
    if (*pC == '\0')
      *pDpoint=0;
  }

  strncpy (szOut, pStart, nCnt); 
}

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rssgu_invalidViBooleanRange (ViSession instrSession, ViBoolean val)
{
    rssgu_instrRange    instrPtr;
    ViStatus rssgu_status = VI_SUCCESS;
    unsigned int        ptrHandle = 0xffffffff;

    rssgu_status = rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle);
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);


    if ((rssgu_status == VI_SUCCESS) && (instrPtr->rangeChecking == VI_TRUE) )
        return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
    else
        return VI_FALSE;

}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rssgu_invalidViInt32Range (ViSession instrSession, ViInt32 val, ViInt32 min, ViInt32 max)
{
    rssgu_instrRange    instrPtr;
    ViStatus            rssgu_status = VI_SUCCESS;
    unsigned int        ptrHandle = 0xffffffff;

    rssgu_status = rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle);
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    if ((rssgu_status == VI_SUCCESS) && (instrPtr->rangeChecking == VI_TRUE) )
        return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
    else
        return VI_FALSE;
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rssgu_invalidViReal64Range (ViSession instrSession, ViReal64 val, ViReal64 min, ViReal64 max)
{
    rssgu_instrRange    instrPtr;
    ViStatus            rssgu_status = VI_SUCCESS;
    unsigned int        ptrHandle = 0xffffffff;

    rssgu_status = rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle);
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    if ((rssgu_status == VI_SUCCESS) && (instrPtr->rangeChecking == VI_TRUE) )
        return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
    else
        return VI_FALSE;
}
/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rssgu_checkStatus (ViSession instrSession)
{
    ViStatus            rssgu_status = VI_SUCCESS;
    rssgu_instrRange    instrPtr = NULL;
    unsigned int        ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> errorChecking != VI_FALSE)
        {
            ViInt32 esrByte = 0;

            /*--------------- Read the Standard Event Register ------------------*/
            CHECKERR (rsQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));

            /*---------------- Check if any error or message bit was asserted ---*/
            rssgu_status = ((esrByte & IEEE_ERROR_BITS) != 0) ? RSSGU_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }
    }

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rssgu_waitOPC (ViSession instrSession, ViPString command,
                ViPString instrResp, ViInt32 timeout)
{
    ViStatus    rssgu_status    = VI_SUCCESS;
    ViUInt16    response        = 0;
    ViInt32     counter         = 0;


    CHECKERR (rsPrintf (instrSession, "%s\n", command));

    while ((response & 0x10) == 0) /* (queries MAV in STB) */
        {

        CHECKERR (rsReadSTB (instrSession, &response));

        (void) rssgu_delay (instrSession, 0.005);
        counter+= 5;
        if (counter >= timeout)
            return VI_ERROR_TMO;
        }

    CHECKERR (rsScanf (instrSession, "%[^\r\n]", instrResp));

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rssgu_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rssgu_status    = VI_SUCCESS;
    size_t      length          = 0;
    ViChar      *pbuffer;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc(length + 1);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;

    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    length++;
    GOCHECKERR (rsWrite (instrSession, (ViPBuf) pbuffer, (ViUInt32) length, VI_NULL));
    GOCHECKERR (rssgu_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssgu_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      *wrt_buf;
    ViStatus    rssgu_status = VI_SUCCESS;

    if ((max - min) > 0)
        if (rssgu_invalidViInt32Range (instrSession, value, min, max) == VI_TRUE)
            return parameterErrorCode;

    
    wrt_buf = (ViChar*) malloc (strlen (cmd) + 20);
    
    sprintf (wrt_buf, "%s %ld", cmd, value);

    rssgu_status = rssgu_sendCmd (instrSession, wrt_buf);
    
    free (wrt_buf);
    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssgu_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( rsPrintf(instrSession, "%s\n", cmd));
    CHECKERR( rsScanf(instrSession, "%ld", value));
    CHECKERR( rssgu_checkStatus (instrSession));

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssgu_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    if (rssgu_invalidViBooleanRange (instrSession, value) == VI_TRUE)
        return parameterErrorCode;

    return rssgu_setViString (instrSession,
                              cmd,
                              (value == VI_TRUE) ? "ON" : "OFF",
                              0);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssgu_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status;
    ViChar      buffer[4];

    if (value == NULL)
        return parameterErrorCode;

    CHECKERR (rssgu_getViString (instrSession,
                                cmd,
                                buffer, 4,
                                0));

    *value = (ViBoolean) ((strncmp (buffer, "1", 1) == 0) || (strncmp (buffer,"ON",2) == 0));

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rssgu_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer1[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rssgu_invalidViReal64Range (instrSession, value, min, max) == VI_TRUE)
            return parameterErrorCode;

    f2a (value, buffer1,BUFFER_SIZE);
    sprintf(buffer,"%s %s", cmd, buffer1);
    return rssgu_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rssgu_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
  ViStatus    rssgu_status = VI_SUCCESS;
  ViChar      buffer[BUFFER_SIZE];

  if ( value == VI_NULL )
    return parameterErrorCode;

  CHECKERR (rssgu_getViString (instrSession,
    cmd,
    buffer, BUFFER_SIZE,
    0));
  sscanf(buffer, "%le", value);
  CHECKERR(rssgu_checkStatus (instrSession));
  return rssgu_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 with Units                                         */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rssgu_setViReal64Units (ViSession instrSession,
                                 ViString cmd,
                                 ViReal64 value,
                                 ViString units,
                                 ViReal64 min,
                                 ViReal64 max,
                                 ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rssgu_invalidViReal64Range (instrSession, value, min, max) == VI_TRUE)
            return parameterErrorCode;

    sprintf(buffer,"%s %.12f %s",cmd, value, units);

    return rssgu_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rssgu_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(rsWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rssgu_checkStatus(instrSession));

ERROR_LABEL:

    free(wrt_buf);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rssgu_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count           = 0;
    size_t      length          = 0;

    if (value == NULL)
        return parameterErrorCode;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    memcpy (pbuffer, cmd, length);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    pbuffer[length] = '\n';

    GOCHECKERR (rsWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free (pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssgu_readDataUnknownLength (instrSession, &pbuffer, &count));
    
    if (pbuffer == NULL)
        return rssgu_status;
    
    GOCHECKERR (rssgu_checkStatus (instrSession));

    if (count > buffer_size)
    {
        rssgu_status = RSSGU_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, (size_t) count);
        value[count] = '\0';
    }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
        {
            rssgu_status = VI_SUCCESS;
            memcpy (value, pbuffer, (size_t) (p2buffer - pbuffer));
            value[p2buffer - pbuffer] = '\0';
        }

ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rssgu_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (rsWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rssgu_checkStatus(instrSession));

ERROR_LABEL:

    free (wrt_buf);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get Quoted String                                               */
/* Purpose:  This function gets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rssgu_getViStringQuoted (ViSession instrSession,
                                  ViString cmd,
                                  ViString quotes,
                                  ViInt32  arraySize,
                                  ViChar   value[],
                                  ViStatus parameterErrorValue)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar*     rd_buf;
    ViChar      fmt_string[BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorValue;

    rd_buf = (ViChar*)malloc(arraySize + 2*strlen(quotes) + 4);
    sprintf(fmt_string,"%s%%[^%s]", quotes, quotes);

    {
        ViInt32     length      = 0;
        ViChar      *pbuffer;

        length = (ViInt32)strlen (cmd);
        pbuffer = (ViChar*) malloc (length + 1);
        memcpy (pbuffer, cmd, length);
        pbuffer[length] = '\n';

        rssgu_status = rsWrite (instrSession, (ViPBuf) pbuffer, length + 1, VI_NULL);

        free(pbuffer);
    }

    GOCHECKERR (rssgu_status);
    GOCHECKERR (rsScanf (instrSession, fmt_string, value));
    GOCHECKERR (rssgu_checkStatus(instrSession));

ERROR_LABEL:

    free (rd_buf);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rssgu_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rssgu_status = VI_SUCCESS;

    CHECKERR (rssgu_checkIndexInList (value, list, parameterErrorCode));

    return rssgu_setViString (instrSession, cmd, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rssgu_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE] = "";
    
    CHECKERR( rssgu_getViString(instrSession, cmd, buffer, BUFFER_SIZE, parameterErrorCode));
    
    CHECKERR( rssgu_getIndexFromString (buffer, list, value));

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rssgu_checkIndexInList (ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;

    if (index < 0)
        return statusIfError;

    while (list[i++] != VI_NULL);
    if (index >= i)
        return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rssgu_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViUInt32    i;
    size_t      len;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = (ViInt32) i;

    return (list[i]) ? rssgu_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Get Catalog                                                     */
/*===========================================================================*/
ViStatus rssgu_getCatalog (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViInt32 parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count  = 0;
    size_t      length = 0;

    if (array == NULL)
        return parameterErrorCode;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (rsWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free(pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssgu_readDataUnknownLength (instrSession, &pbuffer, &count));
    GOCHECKERR (rssgu_checkStatus (instrSession));

    if (count > arraySize)
        {
        rssgu_status = RSSGU_WARNING_SMALL_BUFFER;
        count = arraySize;
        memcpy (array, pbuffer, count);
        array[count-1] = '\0';
        }
    else
        {
        if ((p2buffer = strchr (pbuffer, ';')) != VI_NULL)
            {
            rssgu_status = VI_SUCCESS;
            memcpy (array, pbuffer, (size_t)(p2buffer - pbuffer));
            array[p2buffer - pbuffer] = '\0';
            }
        else
            {
            memcpy (array, pbuffer, count);
            array[count-1] = '\0';
            }
        }
ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rssgu_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    GOCHECKERR (rsPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssgu_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rssgu_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get ViInt32 Array                                               */
/*===========================================================================*/
ViStatus rssgu_getViInt32Array (ViSession instrSession,
                            ViString cmd,
                            ViInt32 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    GOCHECKERR (rsPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssgu_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atol(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rssgu_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get ViBoolean Array                                               */
/*===========================================================================*/
ViStatus rssgu_getViBooleanArray (ViSession instrSession,
                            ViString cmd,
                            ViBoolean *value,
                            ViInt32  count,
							ViInt32  *returnedValues, 
                            ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    GOCHECKERR (rsPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssgu_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = (ViBoolean) ((strncmp (p2buf, "1", 1) == 0) || (strncmp (p2buf,"ON",2) == 0)); 
        p2buf = strtok(NULL, ",");
    };

	*returnedValues = i;
		
    GOCHECKERR (rssgu_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial intf                                                 */
/*===========================================================================*/
ViStatus rssgu_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rssgu_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = NULL, *p2buffer = NULL;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, (size_t) sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
    {
        if ((rssgu_status = rsRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
        {
            free(buffer);
            *outBuffer = VI_NULL;
            return rssgu_status;
        }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
    } while (rssgu_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count != NULL) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Get ViUInt32                                                    */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rssgu_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rssgu_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( rsPrintf(instrSession, "%s\n", cmd));
    CHECKERR( rsScanf(instrSession, "%u", value));
    CHECKERR( rssgu_checkStatus (instrSession));

    return rssgu_status;
}

/*****************************************************************************
 * Function: Read Trace Data
 * Purpose:  This function reads out trace data from the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssgu_ReadTraceData(
    ViSession   instrSession,
    ViString    cmd,
    ViInt32*    noofPoints,
    ViReal64    _VI_FAR traceData[]
)
{
    ViUInt32 retCnt = 0;
    ViChar buffer[BUFFER_SIZE];
    ViByte* exBuf;
    ViReal32 *temp;
    ViInt32 lenLength, i = 0;
    ViStatus rssgu_status = VI_SUCCESS;
    ViUInt16 interface = VI_INTF_GPIB;
    rssgu_instrRange instrPtr;
    unsigned int     ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            sprintf (buffer, ":FORMAT ASCII;*CLS;%s\n", cmd);

            CHECKERR (rsWrite (instrSession, (ViBuf) buffer, (ViUInt32)strlen (buffer), &retCnt));
            CHECKERR (rssgu_checkBeforeRead (instrSession));
            CHECKERR (rsScanf (instrSession, "%,#Lf", noofPoints, traceData));
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            sprintf (buffer, ":FORM REAL,32;*CLS;%s\n", cmd);

            if ((rssgu_status = rsWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer),
                &retCnt)) < 0)
                return rssgu_status;
            if ((rssgu_status = rssgu_checkBeforeRead (instrSession)) < 0)
                return rssgu_status;

            if ((rssgu_status = rsRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                return rssgu_status;
            if (sscanf (buffer, "#%d", (int*)&lenLength) != 1) {
                rssgu_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rssgu_status;
            }

            if ((rssgu_status = rsRead (instrSession, (ViBuf) buffer, lenLength,
                &retCnt)) < 0)
                return rssgu_status;
            if (sscanf (buffer, "%ld", noofPoints) != 1) {
                rssgu_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rssgu_status;
            }

            /* Allocate memory space */
            if ((exBuf = (ViByte *)malloc (*noofPoints + 1)) == NULL)
                return VI_ERROR_ALLOC;
            if ((rssgu_status = rsRead (instrSession, (ViBuf) exBuf, *noofPoints + 1,
                &retCnt)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rssgu_status;
                }

            /* Convert binary data into array of ViReal64 types. */
            *noofPoints = (ViInt32)(*noofPoints/4);
            /* Binary data byte order (MOTOROLA or INTEL). */
            rssgu_SWAP_32(exBuf, *noofPoints);
            /* Pointer to data */
            temp = (ViReal32*)(exBuf);
            for (i = 0; i < *noofPoints; i++)
                *traceData++ = (ViReal64)*temp++;

            if (exBuf) /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
        break;
        }

    if ((rssgu_status = rssgu_checkStatus (instrSession)) < 0)
        return rssgu_status;

    return rssgu_status;
}
/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rssgu_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }
}

/*===========================================================================*/
/* Function: Check STB Before Read                                           */
/* Purpose:  This is a utility function which checks Error/Event Queue bit of*/
/*           status byte. Call this function before read result of query     */
/*           command. When error occurs, this function returns error code    */
/*           RSZVB_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE. Prior to calling    */
/*           this function is necessary to clear STB.                        */
/*===========================================================================*/
ViStatus rssgu_checkBeforeRead (ViSession instrSession)
{
    ViStatus rssgu_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViInt32 counter = 0, timeout = 0;
    ViUInt16 interface = VI_INTF_GPIB;
    rssgu_instrRange instrPtr;
    unsigned int        ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    if (instrPtr -> errorChecking) /* Is Error Checking Enabled? */
        {
        /* Interface specific operations */
        switch (interface)
            {
            case VI_INTF_ASRL: /* Serial interface */

                /* Not supported */

            break;
            default: /* Default case */

                CHECKERR (rsGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &timeout));

#if 0 /* Removed from the build */

                /* Result of reading STB must be zero. Initial settings */
                do {
                    CHECKERR (rssgu_status = rsReadSTB (instrSession, &response));

                    rssgu_delay (0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait until STB is non zero value. */
                } while (!response);
#endif
                /* Check STB for errors. */
                do {
                    CHECKERR (rssgu_status = rsReadSTB (instrSession, &response));

                    rssgu_delay (instrSession, 0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait while STB is non zero value. Only Error/Event Queue
                   and MAV bits are taken into consideration. */
                } while (!(response & 0x14));

                if (response & 0x4)
                {
                    ViInt32 error;
                    CHECKERR(rsQueryf(instrSession, "*ESR?\n", "%ld", &error));
                    if (error == 4)
                        CHECKERR(rsQueryf(instrSession, ":SYST:ERR?\n", "%*[^\r\n]"));
                }
                /* (check Error/Event Queue bit in STB) */
                return (response & 0x4) ?  RSSGU_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE : rssgu_status;
            }
        }

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Read Data To File                                               */
/* Purpose:  This function read data from the instrument and                 */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus rssgu_readDataToFile (ViSession instrSession, ViString cmd,
                           ViString destination)
{
    ViStatus    rssgu_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;
    FILE        *targetFile;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (rsWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "%s\n", cmd);

    CHECKERR (rsWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssgu_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssgu_status = rsWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssgu_status;
            }

    if ((rssgu_status = rssgu_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssgu_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != (ViUInt32) fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        (void) fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rssgu_status = rssgu_checkStatus (instrSession)) < 0)
        return rssgu_status;

    return rssgu_status;
}

/*===========================================================================*/
/* Function: Check Instrument                                                */
/* Purpose:  This function checks presence of selected instrument. According */
/*           to generateErrorIfPresent value it either generates error or    */
/*           returns success if instrumet is present.                        */
/*===========================================================================*/
ViStatus rssgu_checkInstrument (ViSession instrSession, ViInt32 instrument, ViBoolean generateErrorIfPresent)
{
    ViStatus            rssgu_status = VI_SUCCESS;
    rssgu_instrRange    instrPtr;
    unsigned int        ptrHandle = 0xffffffff;

    CHECKERR (rsGetAttribute (instrSession, VI_ATTR_USER_DATA, &ptrHandle));
    instrPtr = (rssgu_instrRange)rssgu_GetPointer (ptrHandle);

    if ((instrPtr->errorChecking != 0) && (instrPtr->optionChecking != 0))
    {
        if (generateErrorIfPresent != VI_FALSE)
        {
            if (instrPtr->instrVer == instrument)
                rssgu_status = RSSGU_ERROR_INSTRUMENT_NSUP_MODEL;
        }
        else
        {
            if (instrPtr->instrVer != instrument)
                rssgu_status = RSSGU_ERROR_INSTRUMENT_NSUP_MODEL;
        }
    }

    return rssgu_status;
} 

/*===========================================================================*/
/* Function: String Index                                                    */
/* Purpose:  Returns the index of string in string array                     */
/*===========================================================================*/
ViInt32 rssgu_StringIndex(ViString array[], ViChar buff[])
{
    ViInt32 index = 0;

    while (array[index]){
        if (!strcmp(buff, array[index]))
            break; 
        index++;    
    }        
    /* If found string is not found returns -1. */
    if (!array[index]) return -1;

    return index;
}

